<?php
namespace Woocommerce\Moip\View;

if ( ! function_exists( 'add_action' ) ) {
	exit( 0 );
}

use Woocommerce\Moip\Core;
use Woocommerce\Moip\Helper\Utils;
use Woocommerce\Moip\Model\Order;
use Woocommerce\Moip\Model\Setting;

class Checkouts
{
	protected static function message_before()
	{
		return __( 'Your transaction has been processed by Moip.', Core::TEXTDOMAIN ) . '<br />';
	}

	protected static function message_after()
	{
		return __( 'If you have any questions regarding the transaction, please contact us or the Moip.', Core::TEXTDOMAIN );
	}

	public static function handle_messages( Order $order )
	{
		switch ( $order->payment_type ) {

			case 'payBoleto' :
				return self::billet_message( $order );

			case 'payCreditCard' :
				return self::credit_cart_message( $order );

			case 'payOnlineBankDebitItau' :
				return self::debit_message( $order );
		}
	}

	public static function credit_cart_message( $order )
	{
		$message = self::message_before();

		$message .= sprintf(
			__( 'The status of your transaction is %s and the Moip code is %s.', Core::TEXTDOMAIN ),
			'<strong>' . $order->get_moip_status_translate() . '</strong>',
			'<strong>' . $order->payment_id . '</strong>'
		) . '<br />';

		$message .= self::message_after();

		return $message;
	}

	public static function debit_message( $order )
	{
		$message = self::message_before();

		$message .= __( 'If you have not made ​​the payment, please click the button below to pay.', Core::TEXTDOMAIN ) . '<br />';

		$message .= sprintf(
			'<a href="%s" target="_blank" class="payment-link">%s</a></br>',
			$order->payment_links->payOnlineBankDebitItau->redirectHref,
			__( 'Pay now', Core::TEXTDOMAIN )
		);

		$message .= self::message_after();

		return $message;
	}

	public static function billet_message( $order )
	{
		$message = self::message_before();

		$message .= __( 'If you have not yet received the billet, please click the button below to print.', Core::TEXTDOMAIN ) . '<br />';

		$message .= sprintf(
			'<a href="%s" target="_blank" class="payment-link">%s</a><br/>',
			$order->payment_links->payBoleto->redirectHref,
			__( 'Print', Core::TEXTDOMAIN )
		);

		if ( ! empty( $order->payment_billet_linecode ) ) {
			$message .= sprintf(
				'<span>%s <span id="linecode">%s</span></span><br/>',
				__( 'Line Code: ', Core::TEXTDOMAIN ),
				$order->payment_billet_linecode
			);
		}

		$message .= self::message_after();

		$message .= sprintf(
			'<button id="clipboard-linecode-btn"
			         class="clipboard-btn"
			         data-success-text="%s"
			         data-clipboard-target="#linecode">
			    %s
			 </button>',
			__( 'Copied!', Core::TEXTDOMAIN ),
			__( 'Copy barcode', Core::TEXTDOMAIN )
		);

		return $message;
	}

	public static function render_installments( $wc_order )
	{
		$setting = Setting::get_instance();

		if ( ! $setting->is_active_installments() ) {
			return;
		}

		$max_installments = intval( $setting->installments_maximum );
		$total            = $wc_order->get_total();

		for ( $times = 2; $times <= $max_installments; $times++ ) {

			$amount = $total;

			if ( isset( $setting->installments['interest'][ $times ] ) ) {
				$per_installment = str_replace( ',', '.', $setting->installments['interest'][$times] );
				$interest        = ( $per_installment / 100 ) * $amount;
				$amount         += $interest;
			}

			$price = ceil( $amount / $times * 100 ) / 100;
			$text  = sprintf( __( '%dx of %s (%s)', Core::TEXTDOMAIN ),
				$times,
				wc_price( $price ),
				wc_price( $price * $times )
			);

			printf( '<option value="%1$s">%2$s</option>', $times, $text );
		}
	}
}
