<?php
/*
 * Plugin Name: WooCommerce Moip Transparente
 * Plugin URI:  https://woocommerce.com.br/moip-transparente
 * Version:     1.0.8
 * Author:      John-Henrique
 * Author URI:  https://woocommerce.com.br
 * Text Domain: woocommerce-moip-gateway
 * Domain Path: /languages
 * License:     GPLv3 or later
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 * Description: Integração MoIP para WooCommerce baseada na V2 da API MoIP. Siga as <a href="https://woocommerce.com.br/plugin/moip-plugin-woocommerce/" title="instructions of use" target="_blank">instruções de uso</a>.
 */

if ( ! function_exists( 'add_action' ) ) {
	exit( 0 );
}

define( 'MOIP_TEXTDOMAIN', 'woocommerce-moip-gateway' );
define( 'MOIP_OPTION_ACTIVATE', 'moip_transparente' );
define( 'MOIP_ROOT_FILE', __FILE__ );
define( 'MOIP_VERSION', '1.0.8' );

function wc_moip_render_admin_notice_html( $message, $type = 'error' )
{
?>
	<div class="<?php echo $type; ?> notice is-dismissible">
		<p>
			<strong><?php _e( 'WooCommerce Moip Transparente', MOIP_TEXTDOMAIN ); ?>: </strong>

			<?php echo $message; ?>
		</p>
	</div>
<?php
}

if ( version_compare( PHP_VERSION, '5.6', '<' ) ) {

	function wc_moip_admin_notice_php_version()
	{
		wc_moip_render_admin_notice_html(
			__( 'Your PHP version is not supported. Required >= 5.6.', MOIP_TEXTDOMAIN )
		);
	}

	_wc_moip_load_notice( 'admin_notice_php_version' );
	return;
}

function wc_moip_admin_notice_error()
{
	wc_moip_render_admin_notice_html(
		__( 'WooCoomerce plugin is required.', MOIP_TEXTDOMAIN )
	);
}

function _wc_moip_load_notice( $name )
{
	add_action( 'admin_notices', "wc_moip_{$name}" );
}

function _wc_moip_load_instances()
{
	require_once( 'vendor/autoload.php' );

	Woocommerce\Moip\Core::instance();

	do_action( 'wc_moip_init' );
}

function wc_moip_plugins_loaded_check()
{
	class_exists( 'WooCommerce' ) ? _wc_moip_load_instances() : _wc_moip_load_notice( 'admin_notice_error' );
}
add_action( 'plugins_loaded', 'wc_moip_plugins_loaded_check', 0 );

function wc_moip_on_activation()
{
	if ( ! class_exists( 'WooCommerce' ) ) {
		return;
	}

	add_option( MOIP_OPTION_ACTIVATE, true );

	register_uninstall_hook( __FILE__, 'wc_moip_on_uninstall' );
}

function wc_moip_on_deactivation()
{

}

function wc_moip_on_uninstall()
{

}

register_activation_hook( __FILE__, 'wc_moip_on_activation' );
register_deactivation_hook( __FILE__, 'wc_moip_on_deactivation' );
