;(function(context, $) {

    'use strict';

    var MONSTER = function(namespace, callback) {
        var parts  = namespace.split( '\.' )
          , parent = context
          , count  = parts.length
          , i      = 0
        ;

        for ( i; i < count; i++ ) {
            parent[parts[i]] = ( count - 1 === i ) ? MONSTER.Builder() : parent[parts[i]] || {};
            parent           = parent[parts[i]];
        }

        if ( 'function' === typeof callback ) {

            if ( !~namespace.indexOf( 'Components' ) ) {
                parent.click = MONSTER.click.bind( parent );
                parent.ajax  = MONSTER.ajax.bind( parent );
            }

            callback.call( null, parent, $, MONSTER.utils );
        }

        return parent;
    };

    MONSTER.getElements = function(context) {
        var elements  = {}
          , byElement = this.byElement.bind( context )
        ;

        context.find( '[data-element]' ).each(function(index, element) {
            var name = this.utils.toDataSetName( element.dataset.element );

            if ( !elements[name] ) {
                elements[name] = byElement( element.dataset.element );
            }
        }.bind( this ) );

        return elements;
    };

    MONSTER.byAction = function(name, el) {
        var container = ( el || this );
        return container.find( '[data-action="' + name + '"]' );
    };

    MONSTER.byElement = function(name) {
        return this.find( '[data-element="' + name + '"]' );
    };

    MONSTER.event = function(instance, event, action) {
        var handle = MONSTER.utils.getEventCallbackName( action, event );
        this.byAction( action ).on( event, $.proxy( instance, handle ) );
    };

    MONSTER.getInstance = function(instance, context) {
        context.byAction   = this.byAction.bind( context );
        instance.$el       = context;
        instance.data      = context.data();
        instance.on        = this.event.bind( context, instance );
        instance.elements  = this.getElements( context );
        instance.addPrefix = this.utils.addPrefix;
        instance.prefix    = this.utils.prefix();
        instance.ajax      = this.ajax.bind( instance );
        instance.click     = this.click.bind( instance );
        instance.body      = MOIP.vars.body;

        return instance;
    };

    MONSTER.Builder = function() {
        var Kernel, Builder;
        var self = this;

        Kernel  = function() {};
        Builder = function(context) {
            var instance = new Kernel();
            instance     = self.getInstance( instance, context );

            instance.start.apply( instance, arguments );

            return instance;
        };

        Builder.fn       = Builder.prototype;
        Kernel.prototype = Builder.fn;
        Builder.fn.start = function() {};

        return Builder;
    };

    MONSTER.ajax = function(options, done, fail) {
        var ajax
          , defaults = {
            method : 'POST',
            url    : MONSTER.utils.getAjaxUrl(),
            data   : {}
        };

        ajax = $.ajax( $.extend( defaults, ( options || {} ) ) );

        ajax.done( $.proxy( this, ( done || '_done' ) ) );
        ajax.fail( $.proxy( this, ( fail || '_fail' ) ) );
    };

    MONSTER.click = function(action, context) {
        var instance = ( context || this );
        MONSTER.byAction( action, instance.$el ).on( 'click', $.proxy( instance, MONSTER.utils.getEventCallbackName( action ) ) );
    };

    MONSTER.utils = {

        getGlobalVars: function(name) {
            return ( window.MOIPGlobalVars || {} )[name];
        },

        prefix: function() {
            return ( this.getGlobalVars( 'prefix' ) || 'monster' )
        },

        getAjaxUrl: function() {
            return this.getGlobalVars( 'ajaxUrl' );
        },

        getLocale: function() {
            return this.getGlobalVars( 'WPLANG' );
        },

        getSpinnerUrl: function() {
            return this.getGlobalVars( 'spinnerUrl' );
        },

        getPathUrl: function(url) {
            return decodeURIComponent( url ).split(/[?#]/)[0];
        },

        getTime: function() {
            return ( new Date() ).getTime();
        },

        encodeUrl: function(url) {
            return encodeURIComponent( url );
        },

        decodeUrl: function(url) {
            return decodeURIComponent( url );
        },

        ucfirst: function(text) {
            return this.parseName( text, /(\b[a-z])/g );
        },

        toDataSetName: function(text) {
            return this.parseName( text, /(-)\w/g );
        },

        hasParam: function() {
            return ~window.location.href.indexOf( '?' );
        },

        getPathName: function() {
            return window.location.pathname;
        },

        getEventCallbackName: function(action, event) {
            return this.ucfirst( [ '_on', ( event || 'click' ), action ].join( '-' ) );
        },

        addPrefix: function(tag, separator) {
            var sep = ( separator || '-' );
            return this.prefix() + sep + tag;
        },

        getSpinner: function() {
            var img       = document.createElement( 'img' );
            img.src       = this.getSpinnerUrl();
            img.className = this.prefix() + '-spinner';

            return img;
        },

        isMobile: function() {
            return ( /Android|webOS|iPhone|iPad|iPod|BlackBerry|Tablet OS|IEMobile|Opera Mini/i.test(
                navigator.userAgent
            ) );
        },

        parseName: function(text, regex) {
            return text.replace( regex, function(match) {
                return match.toUpperCase();
            }).replace( /-/g, '' );
        },

        remove: function(element) {
            element.fadeOut( 'fast', function() {
                element.remove();
            });
        },

        getId: function(id) {
            if ( !id ) {
                return false;
            }

            return document.getElementById( id );
        },

        findComponent: function(selector, callback) {
            var components = $(this).find( selector );

            if ( components.length && typeof callback === 'function' ) {
                callback.call( null, components, $(this) );
            }

            return components.length;
        },

        get: function(key, defaultVal) {
            var query, vars, varsLength, pair, i;

            if ( !this.hasParam() ) {
                return ( defaultVal || '' );
            }

            query      = window.location.search.substring(1);
            vars       = query.split( '&' );
            varsLength = vars.length;

            for ( i = 0; i < varsLength; i++ ) {
                pair = vars[i].split( '=' );

                if ( pair[0] === key ) {
                    return pair[1];
                }
            }

            return ( defaultVal || '' );
        },

        strToCode: function(str) {
            var hash   = 0
              , strLen = str.length
              , i
              , chr
            ;

            if ( !strLen ) {
                return hash;
            }

            for ( i = 0; i < strLen; i++ ) {
                chr   = str.charCodeAt( i );
                hash  = ( ( hash << 5 ) - hash ) + chr;
                hash |= 0;
            }

            return Math.abs( hash );
        }
    };

    context.MONSTER = MONSTER;

})( window, jQuery );;MONSTER( 'MOIP.BuildComponents', function(Model, $, utils) {

	Model.create = function(container) {
		var components    = '[data-' + utils.addPrefix( 'component' ) + ']'
		  , findComponent = utils.findComponent.bind( container )
		;

		findComponent( components, $.proxy( this, '_start' ) );
	};

	Model._start = function(components) {
		if ( typeof MOIP.Components === 'undefined' ) {
			return;
		}

		this._iterator( components );
	};

	Model._iterator = function(components) {
		var name;

		components.each( function(index, component) {
			component = $( component );
			name      = utils.ucfirst( this.getComponent( component ) );
			this._callback( name, component );
		}.bind( this ) );
	};

	Model.getComponent = function(component) {
		var component = component.data( utils.addPrefix( 'component' ) );

		if ( !component ) {
			return '';
		}

		return component;
	};

	Model._callback = function(name, component) {
		var callback = MOIP.Components[name];

		if ( typeof callback == 'function' ) {
			callback.call( null, component );
			return;
		}

		console.log( 'Component "' + name + '" is not a function.' );
	};

}, {} );;MONSTER( 'MOIP.BuildCreate', function(Model, $, utils) {

	Model.init = function(container, names) {
		if ( !names.length ) {
			return;
		}

		this.$el = container;
		names.forEach( this.findNames.bind( this ) );
	};

	Model.findNames = function(name, index) {
		this.callback( MOIP[utils.ucfirst( name )] );
	};

	Model.callback = function(callback) {
		if ( typeof callback !== 'function' ) {
			return;
		}

		callback.create( this.$el );
	};

}, {} );;/**
 * jquery.mask.js
 * @version: v1.14.10
 * @author: Igor Escobar
 *
 * Created by Igor Escobar on 2012-03-10. Please report any bug at http://blog.igorescobar.com
 *
 * Copyright (c) 2012 Igor Escobar http://blog.igorescobar.com
 *
 * The MIT License (http://www.opensource.org/licenses/mit-license.php)
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

/* jshint laxbreak: true */
/* jshint maxcomplexity:17 */
/* global define */

'use strict';

// UMD (Universal Module Definition) patterns for JavaScript modules that work everywhere.
// https://github.com/umdjs/umd/blob/master/jqueryPluginCommonjs.js
(function (factory, jQuery, Zepto) {

    if (typeof define === 'function' && define.amd) {
        define(['jquery'], factory);
    } else if (typeof exports === 'object') {
        module.exports = factory(require('jquery'));
    } else {
        factory(jQuery || Zepto);
    }

}(function ($) {

    var Mask = function (el, mask, options) {

        var p = {
            invalid: [],
            getCaret: function () {
                try {
                    var sel,
                        pos = 0,
                        ctrl = el.get(0),
                        dSel = document.selection,
                        cSelStart = ctrl.selectionStart;

                    // IE Support
                    if (dSel && navigator.appVersion.indexOf('MSIE 10') === -1) {
                        sel = dSel.createRange();
                        sel.moveStart('character', -p.val().length);
                        pos = sel.text.length;
                    }
                    // Firefox support
                    else if (cSelStart || cSelStart === '0') {
                        pos = cSelStart;
                    }

                    return pos;
                } catch (e) {}
            },
            setCaret: function(pos) {
                try {
                    if (el.is(':focus')) {
                        var range, ctrl = el.get(0);

                        // Firefox, WebKit, etc..
                        if (ctrl.setSelectionRange) {
                            ctrl.setSelectionRange(pos, pos);
                        } else { // IE
                            range = ctrl.createTextRange();
                            range.collapse(true);
                            range.moveEnd('character', pos);
                            range.moveStart('character', pos);
                            range.select();
                        }
                    }
                } catch (e) {}
            },
            events: function() {
                el
                .on('keydown.mask', function(e) {
                    el.data('mask-keycode', e.keyCode || e.which);
                    el.data('mask-previus-value', el.val());
                })
                .on($.jMaskGlobals.useInput ? 'input.mask' : 'keyup.mask', p.behaviour)
                .on('paste.mask drop.mask', function() {
                    setTimeout(function() {
                        el.keydown().keyup();
                    }, 100);
                })
                .on('change.mask', function(){
                    el.data('changed', true);
                })
                .on('blur.mask', function(){
                    if (oldValue !== p.val() && !el.data('changed')) {
                        el.trigger('change');
                    }
                    el.data('changed', false);
                })
                // it's very important that this callback remains in this position
                // otherwhise oldValue it's going to work buggy
                .on('blur.mask', function() {
                    oldValue = p.val();
                })
                // select all text on focus
                .on('focus.mask', function (e) {
                    if (options.selectOnFocus === true) {
                        $(e.target).select();
                    }
                })
                // clear the value if it not complete the mask
                .on('focusout.mask', function() {
                    if (options.clearIfNotMatch && !regexMask.test(p.val())) {
                       p.val('');
                   }
                });
            },
            getRegexMask: function() {
                var maskChunks = [], translation, pattern, optional, recursive, oRecursive, r;

                for (var i = 0; i < mask.length; i++) {
                    translation = jMask.translation[mask.charAt(i)];

                    if (translation) {

                        pattern = translation.pattern.toString().replace(/.{1}$|^.{1}/g, '');
                        optional = translation.optional;
                        recursive = translation.recursive;

                        if (recursive) {
                            maskChunks.push(mask.charAt(i));
                            oRecursive = {digit: mask.charAt(i), pattern: pattern};
                        } else {
                            maskChunks.push(!optional && !recursive ? pattern : (pattern + '?'));
                        }

                    } else {
                        maskChunks.push(mask.charAt(i).replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&'));
                    }
                }

                r = maskChunks.join('');

                if (oRecursive) {
                    r = r.replace(new RegExp('(' + oRecursive.digit + '(.*' + oRecursive.digit + ')?)'), '($1)?')
                         .replace(new RegExp(oRecursive.digit, 'g'), oRecursive.pattern);
                }

                return new RegExp(r);
            },
            destroyEvents: function() {
                el.off(['input', 'keydown', 'keyup', 'paste', 'drop', 'blur', 'focusout', ''].join('.mask '));
            },
            val: function(v) {
                var isInput = el.is('input'),
                    method = isInput ? 'val' : 'text',
                    r;

                if (arguments.length > 0) {
                    if (el[method]() !== v) {
                        el[method](v);
                    }
                    r = el;
                } else {
                    r = el[method]();
                }

                return r;
            },
            calculateCaretPosition: function(caretPos, newVal) {
                var newValL = newVal.length,
                    oValue  = el.data('mask-previus-value') || '',
                    oValueL = oValue.length;

                // edge cases when erasing digits
                if (el.data('mask-keycode') === 8 && oValue !== newVal) {
                    caretPos = caretPos - (newVal.slice(0, caretPos).length - oValue.slice(0, caretPos).length);

                // edge cases when typing new digits
                } else if (oValue !== newVal) {
                    // if the cursor is at the end keep it there
                    if (caretPos >= oValueL) {
                        caretPos = newValL;
                    } else {
                        caretPos = caretPos + (newVal.slice(0, caretPos).length - oValue.slice(0, caretPos).length);
                    }
                }

                return caretPos;
            },
            behaviour: function(e) {
                e = e || window.event;
                p.invalid = [];

                var keyCode = el.data('mask-keycode');

                if ($.inArray(keyCode, jMask.byPassKeys) === -1) {
                    var newVal   = p.getMasked(),
                        caretPos = p.getCaret();

                    setTimeout(function(caretPos, newVal) {
                      p.setCaret(p.calculateCaretPosition(caretPos, newVal));
                    }, 10, caretPos, newVal);

                    p.val(newVal);
                    p.setCaret(caretPos);
                    return p.callbacks(e);
                }
            },
            getMasked: function(skipMaskChars, val) {
                var buf = [],
                    value = val === undefined ? p.val() : val + '',
                    m = 0, maskLen = mask.length,
                    v = 0, valLen = value.length,
                    offset = 1, addMethod = 'push',
                    resetPos = -1,
                    lastMaskChar,
                    check;

                if (options.reverse) {
                    addMethod = 'unshift';
                    offset = -1;
                    lastMaskChar = 0;
                    m = maskLen - 1;
                    v = valLen - 1;
                    check = function () {
                        return m > -1 && v > -1;
                    };
                } else {
                    lastMaskChar = maskLen - 1;
                    check = function () {
                        return m < maskLen && v < valLen;
                    };
                }

                var lastUntranslatedMaskChar;
                while (check()) {
                    var maskDigit = mask.charAt(m),
                        valDigit = value.charAt(v),
                        translation = jMask.translation[maskDigit];

                    if (translation) {
                        if (valDigit.match(translation.pattern)) {
                            buf[addMethod](valDigit);
                             if (translation.recursive) {
                                if (resetPos === -1) {
                                    resetPos = m;
                                } else if (m === lastMaskChar) {
                                    m = resetPos - offset;
                                }

                                if (lastMaskChar === resetPos) {
                                    m -= offset;
                                }
                            }
                            m += offset;
                        } else if (valDigit === lastUntranslatedMaskChar) {
                            // matched the last untranslated (raw) mask character that we encountered
                            // likely an insert offset the mask character from the last entry; fall
                            // through and only increment v
                            lastUntranslatedMaskChar = undefined;
                        } else if (translation.optional) {
                            m += offset;
                            v -= offset;
                        } else if (translation.fallback) {
                            buf[addMethod](translation.fallback);
                            m += offset;
                            v -= offset;
                        } else {
                          p.invalid.push({p: v, v: valDigit, e: translation.pattern});
                        }
                        v += offset;
                    } else {
                        if (!skipMaskChars) {
                            buf[addMethod](maskDigit);
                        }

                        if (valDigit === maskDigit) {
                            v += offset;
                        } else {
                            lastUntranslatedMaskChar = maskDigit;
                        }

                        m += offset;
                    }
                }

                var lastMaskCharDigit = mask.charAt(lastMaskChar);
                if (maskLen === valLen + 1 && !jMask.translation[lastMaskCharDigit]) {
                    buf.push(lastMaskCharDigit);
                }

                return buf.join('');
            },
            callbacks: function (e) {
                var val = p.val(),
                    changed = val !== oldValue,
                    defaultArgs = [val, e, el, options],
                    callback = function(name, criteria, args) {
                        if (typeof options[name] === 'function' && criteria) {
                            options[name].apply(this, args);
                        }
                    };

                callback('onChange', changed === true, defaultArgs);
                callback('onKeyPress', changed === true, defaultArgs);
                callback('onComplete', val.length === mask.length, defaultArgs);
                callback('onInvalid', p.invalid.length > 0, [val, e, el, p.invalid, options]);
            }
        };

        el = $(el);
        var jMask = this, oldValue = p.val(), regexMask;

        mask = typeof mask === 'function' ? mask(p.val(), undefined, el,  options) : mask;

        // public methods
        jMask.mask = mask;
        jMask.options = options;
        jMask.remove = function() {
            var caret = p.getCaret();
            p.destroyEvents();
            p.val(jMask.getCleanVal());
            p.setCaret(caret);
            return el;
        };

        // get value without mask
        jMask.getCleanVal = function() {
           return p.getMasked(true);
        };

        // get masked value without the value being in the input or element
        jMask.getMaskedVal = function(val) {
           return p.getMasked(false, val);
        };

       jMask.init = function(onlyMask) {
            onlyMask = onlyMask || false;
            options = options || {};

            jMask.clearIfNotMatch  = $.jMaskGlobals.clearIfNotMatch;
            jMask.byPassKeys       = $.jMaskGlobals.byPassKeys;
            jMask.translation      = $.extend({}, $.jMaskGlobals.translation, options.translation);

            jMask = $.extend(true, {}, jMask, options);

            regexMask = p.getRegexMask();

            if (onlyMask) {
                p.events();
                p.val(p.getMasked());
            } else {
                if (options.placeholder) {
                    el.attr('placeholder' , options.placeholder);
                }

                // this is necessary, otherwise if the user submit the form
                // and then press the "back" button, the autocomplete will erase
                // the data. Works fine on IE9+, FF, Opera, Safari.
                if (el.data('mask')) {
                  el.attr('autocomplete', 'off');
                }

                // detect if is necessary let the user type freely.
                // for is a lot faster than forEach.
                for (var i = 0, maxlength = true; i < mask.length; i++) {
                    var translation = jMask.translation[mask.charAt(i)];
                    if (translation && translation.recursive) {
                        maxlength = false;
                        break;
                    }
                }

                if (maxlength) {
                    el.attr('maxlength', mask.length);
                }

                p.destroyEvents();
                p.events();

                var caret = p.getCaret();
                p.val(p.getMasked());
                p.setCaret(caret);
            }
        };

        jMask.init(!el.is('input'));
    };

    $.maskWatchers = {};
    var HTMLAttributes = function () {
        var input = $(this),
            options = {},
            prefix = 'data-mask-',
            mask = input.attr('data-mask');

        if (input.attr(prefix + 'reverse')) {
            options.reverse = true;
        }

        if (input.attr(prefix + 'clearifnotmatch')) {
            options.clearIfNotMatch = true;
        }

        if (input.attr(prefix + 'selectonfocus') === 'true') {
           options.selectOnFocus = true;
        }

        if (notSameMaskObject(input, mask, options)) {
            return input.data('mask', new Mask(this, mask, options));
        }
    },
    notSameMaskObject = function(field, mask, options) {
        options = options || {};
        var maskObject = $(field).data('mask'),
            stringify = JSON.stringify,
            value = $(field).val() || $(field).text();
        try {
            if (typeof mask === 'function') {
                mask = mask(value);
            }
            return typeof maskObject !== 'object' || stringify(maskObject.options) !== stringify(options) || maskObject.mask !== mask;
        } catch (e) {}
    },
    eventSupported = function(eventName) {
        var el = document.createElement('div'), isSupported;

        eventName = 'on' + eventName;
        isSupported = (eventName in el);

        if ( !isSupported ) {
            el.setAttribute(eventName, 'return;');
            isSupported = typeof el[eventName] === 'function';
        }
        el = null;

        return isSupported;
    };

    $.fn.mask = function(mask, options) {
        options = options || {};
        var selector = this.selector,
            globals = $.jMaskGlobals,
            interval = globals.watchInterval,
            watchInputs = options.watchInputs || globals.watchInputs,
            maskFunction = function() {
                if (notSameMaskObject(this, mask, options)) {
                    return $(this).data('mask', new Mask(this, mask, options));
                }
            };

        $(this).each(maskFunction);

        if (selector && selector !== '' && watchInputs) {
            clearInterval($.maskWatchers[selector]);
            $.maskWatchers[selector] = setInterval(function(){
                $(document).find(selector).each(maskFunction);
            }, interval);
        }
        return this;
    };

    $.fn.masked = function(val) {
        return this.data('mask').getMaskedVal(val);
    };

    $.fn.unmask = function() {
        clearInterval($.maskWatchers[this.selector]);
        delete $.maskWatchers[this.selector];
        return this.each(function() {
            var dataMask = $(this).data('mask');
            if (dataMask) {
                dataMask.remove().removeData('mask');
            }
        });
    };

    $.fn.cleanVal = function() {
        return this.data('mask').getCleanVal();
    };

    $.applyDataMask = function(selector) {
        selector = selector || $.jMaskGlobals.maskElements;
        var $selector = (selector instanceof $) ? selector : $(selector);
        $selector.filter($.jMaskGlobals.dataMaskAttr).each(HTMLAttributes);
    };

    var globals = {
        maskElements: 'input,td,span,div',
        dataMaskAttr: '*[data-mask]',
        dataMask: true,
        watchInterval: 300,
        watchInputs: true,
        // old versions of chrome dont work great with input event
        useInput: !/Chrome\/[2-4][0-9]|SamsungBrowser/.test(window.navigator.userAgent) && eventSupported('input'),
        watchDataMask: false,
        byPassKeys: [9, 16, 17, 18, 36, 37, 38, 39, 40, 91],
        translation: {
            '0': {pattern: /\d/},
            '9': {pattern: /\d/, optional: true},
            '#': {pattern: /\d/, recursive: true},
            'A': {pattern: /[a-zA-Z0-9]/},
            'S': {pattern: /[a-zA-Z]/}
        }
    };

    $.jMaskGlobals = $.jMaskGlobals || {};
    globals = $.jMaskGlobals = $.extend(true, {}, globals, $.jMaskGlobals);

    // looking for inputs with data-mask attribute
    if (globals.dataMask) {
        $.applyDataMask();
    }

    setInterval(function() {
        if ($.jMaskGlobals.watchDataMask) {
            $.applyDataMask();
        }
    }, globals.watchInterval);
}, window.jQuery, window.Zepto));
;;(function($) {

	$.fn.isEmptyValue = function() {
		return !( $.trim( this.val() ) );
	};

})( jQuery );;/*!
 * clipboard.js v1.7.1
 * https://zenorocha.github.io/clipboard.js
 *
 * Licensed MIT © Zeno Rocha
 */
(function(f){if(typeof exports==="object"&&typeof module!=="undefined"){module.exports=f()}else if(typeof define==="function"&&define.amd){define([],f)}else{var g;if(typeof window!=="undefined"){g=window}else if(typeof global!=="undefined"){g=global}else if(typeof self!=="undefined"){g=self}else{g=this}g.Clipboard = f()}})(function(){var define,module,exports;return (function e(t,n,r){function s(o,u){if(!n[o]){if(!t[o]){var a=typeof require=="function"&&require;if(!u&&a)return a(o,!0);if(i)return i(o,!0);var f=new Error("Cannot find module '"+o+"'");throw f.code="MODULE_NOT_FOUND",f}var l=n[o]={exports:{}};t[o][0].call(l.exports,function(e){var n=t[o][1][e];return s(n?n:e)},l,l.exports,e,t,n,r)}return n[o].exports}var i=typeof require=="function"&&require;for(var o=0;o<r.length;o++)s(r[o]);return s})({1:[function(require,module,exports){
var DOCUMENT_NODE_TYPE = 9;

/**
 * A polyfill for Element.matches()
 */
if (typeof Element !== 'undefined' && !Element.prototype.matches) {
    var proto = Element.prototype;

    proto.matches = proto.matchesSelector ||
                    proto.mozMatchesSelector ||
                    proto.msMatchesSelector ||
                    proto.oMatchesSelector ||
                    proto.webkitMatchesSelector;
}

/**
 * Finds the closest parent that matches a selector.
 *
 * @param {Element} element
 * @param {String} selector
 * @return {Function}
 */
function closest (element, selector) {
    while (element && element.nodeType !== DOCUMENT_NODE_TYPE) {
        if (typeof element.matches === 'function' &&
            element.matches(selector)) {
          return element;
        }
        element = element.parentNode;
    }
}

module.exports = closest;

},{}],2:[function(require,module,exports){
var closest = require('./closest');

/**
 * Delegates event to a selector.
 *
 * @param {Element} element
 * @param {String} selector
 * @param {String} type
 * @param {Function} callback
 * @param {Boolean} useCapture
 * @return {Object}
 */
function delegate(element, selector, type, callback, useCapture) {
    var listenerFn = listener.apply(this, arguments);

    element.addEventListener(type, listenerFn, useCapture);

    return {
        destroy: function() {
            element.removeEventListener(type, listenerFn, useCapture);
        }
    }
}

/**
 * Finds closest match and invokes callback.
 *
 * @param {Element} element
 * @param {String} selector
 * @param {String} type
 * @param {Function} callback
 * @return {Function}
 */
function listener(element, selector, type, callback) {
    return function(e) {
        e.delegateTarget = closest(e.target, selector);

        if (e.delegateTarget) {
            callback.call(element, e);
        }
    }
}

module.exports = delegate;

},{"./closest":1}],3:[function(require,module,exports){
/**
 * Check if argument is a HTML element.
 *
 * @param {Object} value
 * @return {Boolean}
 */
exports.node = function(value) {
    return value !== undefined
        && value instanceof HTMLElement
        && value.nodeType === 1;
};

/**
 * Check if argument is a list of HTML elements.
 *
 * @param {Object} value
 * @return {Boolean}
 */
exports.nodeList = function(value) {
    var type = Object.prototype.toString.call(value);

    return value !== undefined
        && (type === '[object NodeList]' || type === '[object HTMLCollection]')
        && ('length' in value)
        && (value.length === 0 || exports.node(value[0]));
};

/**
 * Check if argument is a string.
 *
 * @param {Object} value
 * @return {Boolean}
 */
exports.string = function(value) {
    return typeof value === 'string'
        || value instanceof String;
};

/**
 * Check if argument is a function.
 *
 * @param {Object} value
 * @return {Boolean}
 */
exports.fn = function(value) {
    var type = Object.prototype.toString.call(value);

    return type === '[object Function]';
};

},{}],4:[function(require,module,exports){
var is = require('./is');
var delegate = require('delegate');

/**
 * Validates all params and calls the right
 * listener function based on its target type.
 *
 * @param {String|HTMLElement|HTMLCollection|NodeList} target
 * @param {String} type
 * @param {Function} callback
 * @return {Object}
 */
function listen(target, type, callback) {
    if (!target && !type && !callback) {
        throw new Error('Missing required arguments');
    }

    if (!is.string(type)) {
        throw new TypeError('Second argument must be a String');
    }

    if (!is.fn(callback)) {
        throw new TypeError('Third argument must be a Function');
    }

    if (is.node(target)) {
        return listenNode(target, type, callback);
    }
    else if (is.nodeList(target)) {
        return listenNodeList(target, type, callback);
    }
    else if (is.string(target)) {
        return listenSelector(target, type, callback);
    }
    else {
        throw new TypeError('First argument must be a String, HTMLElement, HTMLCollection, or NodeList');
    }
}

/**
 * Adds an event listener to a HTML element
 * and returns a remove listener function.
 *
 * @param {HTMLElement} node
 * @param {String} type
 * @param {Function} callback
 * @return {Object}
 */
function listenNode(node, type, callback) {
    node.addEventListener(type, callback);

    return {
        destroy: function() {
            node.removeEventListener(type, callback);
        }
    }
}

/**
 * Add an event listener to a list of HTML elements
 * and returns a remove listener function.
 *
 * @param {NodeList|HTMLCollection} nodeList
 * @param {String} type
 * @param {Function} callback
 * @return {Object}
 */
function listenNodeList(nodeList, type, callback) {
    Array.prototype.forEach.call(nodeList, function(node) {
        node.addEventListener(type, callback);
    });

    return {
        destroy: function() {
            Array.prototype.forEach.call(nodeList, function(node) {
                node.removeEventListener(type, callback);
            });
        }
    }
}

/**
 * Add an event listener to a selector
 * and returns a remove listener function.
 *
 * @param {String} selector
 * @param {String} type
 * @param {Function} callback
 * @return {Object}
 */
function listenSelector(selector, type, callback) {
    return delegate(document.body, selector, type, callback);
}

module.exports = listen;

},{"./is":3,"delegate":2}],5:[function(require,module,exports){
function select(element) {
    var selectedText;

    if (element.nodeName === 'SELECT') {
        element.focus();

        selectedText = element.value;
    }
    else if (element.nodeName === 'INPUT' || element.nodeName === 'TEXTAREA') {
        var isReadOnly = element.hasAttribute('readonly');

        if (!isReadOnly) {
            element.setAttribute('readonly', '');
        }

        element.select();
        element.setSelectionRange(0, element.value.length);

        if (!isReadOnly) {
            element.removeAttribute('readonly');
        }

        selectedText = element.value;
    }
    else {
        if (element.hasAttribute('contenteditable')) {
            element.focus();
        }

        var selection = window.getSelection();
        var range = document.createRange();

        range.selectNodeContents(element);
        selection.removeAllRanges();
        selection.addRange(range);

        selectedText = selection.toString();
    }

    return selectedText;
}

module.exports = select;

},{}],6:[function(require,module,exports){
function E () {
  // Keep this empty so it's easier to inherit from
  // (via https://github.com/lipsmack from https://github.com/scottcorgan/tiny-emitter/issues/3)
}

E.prototype = {
  on: function (name, callback, ctx) {
    var e = this.e || (this.e = {});

    (e[name] || (e[name] = [])).push({
      fn: callback,
      ctx: ctx
    });

    return this;
  },

  once: function (name, callback, ctx) {
    var self = this;
    function listener () {
      self.off(name, listener);
      callback.apply(ctx, arguments);
    };

    listener._ = callback
    return this.on(name, listener, ctx);
  },

  emit: function (name) {
    var data = [].slice.call(arguments, 1);
    var evtArr = ((this.e || (this.e = {}))[name] || []).slice();
    var i = 0;
    var len = evtArr.length;

    for (i; i < len; i++) {
      evtArr[i].fn.apply(evtArr[i].ctx, data);
    }

    return this;
  },

  off: function (name, callback) {
    var e = this.e || (this.e = {});
    var evts = e[name];
    var liveEvents = [];

    if (evts && callback) {
      for (var i = 0, len = evts.length; i < len; i++) {
        if (evts[i].fn !== callback && evts[i].fn._ !== callback)
          liveEvents.push(evts[i]);
      }
    }

    // Remove event from queue to prevent memory leak
    // Suggested by https://github.com/lazd
    // Ref: https://github.com/scottcorgan/tiny-emitter/commit/c6ebfaa9bc973b33d110a84a307742b7cf94c953#commitcomment-5024910

    (liveEvents.length)
      ? e[name] = liveEvents
      : delete e[name];

    return this;
  }
};

module.exports = E;

},{}],7:[function(require,module,exports){
(function (global, factory) {
    if (typeof define === "function" && define.amd) {
        define(['module', 'select'], factory);
    } else if (typeof exports !== "undefined") {
        factory(module, require('select'));
    } else {
        var mod = {
            exports: {}
        };
        factory(mod, global.select);
        global.clipboardAction = mod.exports;
    }
})(this, function (module, _select) {
    'use strict';

    var _select2 = _interopRequireDefault(_select);

    function _interopRequireDefault(obj) {
        return obj && obj.__esModule ? obj : {
            default: obj
        };
    }

    var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
        return typeof obj;
    } : function (obj) {
        return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };

    function _classCallCheck(instance, Constructor) {
        if (!(instance instanceof Constructor)) {
            throw new TypeError("Cannot call a class as a function");
        }
    }

    var _createClass = function () {
        function defineProperties(target, props) {
            for (var i = 0; i < props.length; i++) {
                var descriptor = props[i];
                descriptor.enumerable = descriptor.enumerable || false;
                descriptor.configurable = true;
                if ("value" in descriptor) descriptor.writable = true;
                Object.defineProperty(target, descriptor.key, descriptor);
            }
        }

        return function (Constructor, protoProps, staticProps) {
            if (protoProps) defineProperties(Constructor.prototype, protoProps);
            if (staticProps) defineProperties(Constructor, staticProps);
            return Constructor;
        };
    }();

    var ClipboardAction = function () {
        /**
         * @param {Object} options
         */
        function ClipboardAction(options) {
            _classCallCheck(this, ClipboardAction);

            this.resolveOptions(options);
            this.initSelection();
        }

        /**
         * Defines base properties passed from constructor.
         * @param {Object} options
         */


        _createClass(ClipboardAction, [{
            key: 'resolveOptions',
            value: function resolveOptions() {
                var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

                this.action = options.action;
                this.container = options.container;
                this.emitter = options.emitter;
                this.target = options.target;
                this.text = options.text;
                this.trigger = options.trigger;

                this.selectedText = '';
            }
        }, {
            key: 'initSelection',
            value: function initSelection() {
                if (this.text) {
                    this.selectFake();
                } else if (this.target) {
                    this.selectTarget();
                }
            }
        }, {
            key: 'selectFake',
            value: function selectFake() {
                var _this = this;

                var isRTL = document.documentElement.getAttribute('dir') == 'rtl';

                this.removeFake();

                this.fakeHandlerCallback = function () {
                    return _this.removeFake();
                };
                this.fakeHandler = this.container.addEventListener('click', this.fakeHandlerCallback) || true;

                this.fakeElem = document.createElement('textarea');
                // Prevent zooming on iOS
                this.fakeElem.style.fontSize = '12pt';
                // Reset box model
                this.fakeElem.style.border = '0';
                this.fakeElem.style.padding = '0';
                this.fakeElem.style.margin = '0';
                // Move element out of screen horizontally
                this.fakeElem.style.position = 'absolute';
                this.fakeElem.style[isRTL ? 'right' : 'left'] = '-9999px';
                // Move element to the same position vertically
                var yPosition = window.pageYOffset || document.documentElement.scrollTop;
                this.fakeElem.style.top = yPosition + 'px';

                this.fakeElem.setAttribute('readonly', '');
                this.fakeElem.value = this.text;

                this.container.appendChild(this.fakeElem);

                this.selectedText = (0, _select2.default)(this.fakeElem);
                this.copyText();
            }
        }, {
            key: 'removeFake',
            value: function removeFake() {
                if (this.fakeHandler) {
                    this.container.removeEventListener('click', this.fakeHandlerCallback);
                    this.fakeHandler = null;
                    this.fakeHandlerCallback = null;
                }

                if (this.fakeElem) {
                    this.container.removeChild(this.fakeElem);
                    this.fakeElem = null;
                }
            }
        }, {
            key: 'selectTarget',
            value: function selectTarget() {
                this.selectedText = (0, _select2.default)(this.target);
                this.copyText();
            }
        }, {
            key: 'copyText',
            value: function copyText() {
                var succeeded = void 0;

                try {
                    succeeded = document.execCommand(this.action);
                } catch (err) {
                    succeeded = false;
                }

                this.handleResult(succeeded);
            }
        }, {
            key: 'handleResult',
            value: function handleResult(succeeded) {
                this.emitter.emit(succeeded ? 'success' : 'error', {
                    action: this.action,
                    text: this.selectedText,
                    trigger: this.trigger,
                    clearSelection: this.clearSelection.bind(this)
                });
            }
        }, {
            key: 'clearSelection',
            value: function clearSelection() {
                if (this.trigger) {
                    this.trigger.focus();
                }

                window.getSelection().removeAllRanges();
            }
        }, {
            key: 'destroy',
            value: function destroy() {
                this.removeFake();
            }
        }, {
            key: 'action',
            set: function set() {
                var action = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'copy';

                this._action = action;

                if (this._action !== 'copy' && this._action !== 'cut') {
                    throw new Error('Invalid "action" value, use either "copy" or "cut"');
                }
            },
            get: function get() {
                return this._action;
            }
        }, {
            key: 'target',
            set: function set(target) {
                if (target !== undefined) {
                    if (target && (typeof target === 'undefined' ? 'undefined' : _typeof(target)) === 'object' && target.nodeType === 1) {
                        if (this.action === 'copy' && target.hasAttribute('disabled')) {
                            throw new Error('Invalid "target" attribute. Please use "readonly" instead of "disabled" attribute');
                        }

                        if (this.action === 'cut' && (target.hasAttribute('readonly') || target.hasAttribute('disabled'))) {
                            throw new Error('Invalid "target" attribute. You can\'t cut text from elements with "readonly" or "disabled" attributes');
                        }

                        this._target = target;
                    } else {
                        throw new Error('Invalid "target" value, use a valid Element');
                    }
                }
            },
            get: function get() {
                return this._target;
            }
        }]);

        return ClipboardAction;
    }();

    module.exports = ClipboardAction;
});

},{"select":5}],8:[function(require,module,exports){
(function (global, factory) {
    if (typeof define === "function" && define.amd) {
        define(['module', './clipboard-action', 'tiny-emitter', 'good-listener'], factory);
    } else if (typeof exports !== "undefined") {
        factory(module, require('./clipboard-action'), require('tiny-emitter'), require('good-listener'));
    } else {
        var mod = {
            exports: {}
        };
        factory(mod, global.clipboardAction, global.tinyEmitter, global.goodListener);
        global.clipboard = mod.exports;
    }
})(this, function (module, _clipboardAction, _tinyEmitter, _goodListener) {
    'use strict';

    var _clipboardAction2 = _interopRequireDefault(_clipboardAction);

    var _tinyEmitter2 = _interopRequireDefault(_tinyEmitter);

    var _goodListener2 = _interopRequireDefault(_goodListener);

    function _interopRequireDefault(obj) {
        return obj && obj.__esModule ? obj : {
            default: obj
        };
    }

    var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
        return typeof obj;
    } : function (obj) {
        return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };

    function _classCallCheck(instance, Constructor) {
        if (!(instance instanceof Constructor)) {
            throw new TypeError("Cannot call a class as a function");
        }
    }

    var _createClass = function () {
        function defineProperties(target, props) {
            for (var i = 0; i < props.length; i++) {
                var descriptor = props[i];
                descriptor.enumerable = descriptor.enumerable || false;
                descriptor.configurable = true;
                if ("value" in descriptor) descriptor.writable = true;
                Object.defineProperty(target, descriptor.key, descriptor);
            }
        }

        return function (Constructor, protoProps, staticProps) {
            if (protoProps) defineProperties(Constructor.prototype, protoProps);
            if (staticProps) defineProperties(Constructor, staticProps);
            return Constructor;
        };
    }();

    function _possibleConstructorReturn(self, call) {
        if (!self) {
            throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
        }

        return call && (typeof call === "object" || typeof call === "function") ? call : self;
    }

    function _inherits(subClass, superClass) {
        if (typeof superClass !== "function" && superClass !== null) {
            throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
        }

        subClass.prototype = Object.create(superClass && superClass.prototype, {
            constructor: {
                value: subClass,
                enumerable: false,
                writable: true,
                configurable: true
            }
        });
        if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
    }

    var Clipboard = function (_Emitter) {
        _inherits(Clipboard, _Emitter);

        /**
         * @param {String|HTMLElement|HTMLCollection|NodeList} trigger
         * @param {Object} options
         */
        function Clipboard(trigger, options) {
            _classCallCheck(this, Clipboard);

            var _this = _possibleConstructorReturn(this, (Clipboard.__proto__ || Object.getPrototypeOf(Clipboard)).call(this));

            _this.resolveOptions(options);
            _this.listenClick(trigger);
            return _this;
        }

        /**
         * Defines if attributes would be resolved using internal setter functions
         * or custom functions that were passed in the constructor.
         * @param {Object} options
         */


        _createClass(Clipboard, [{
            key: 'resolveOptions',
            value: function resolveOptions() {
                var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

                this.action = typeof options.action === 'function' ? options.action : this.defaultAction;
                this.target = typeof options.target === 'function' ? options.target : this.defaultTarget;
                this.text = typeof options.text === 'function' ? options.text : this.defaultText;
                this.container = _typeof(options.container) === 'object' ? options.container : document.body;
            }
        }, {
            key: 'listenClick',
            value: function listenClick(trigger) {
                var _this2 = this;

                this.listener = (0, _goodListener2.default)(trigger, 'click', function (e) {
                    return _this2.onClick(e);
                });
            }
        }, {
            key: 'onClick',
            value: function onClick(e) {
                var trigger = e.delegateTarget || e.currentTarget;

                if (this.clipboardAction) {
                    this.clipboardAction = null;
                }

                this.clipboardAction = new _clipboardAction2.default({
                    action: this.action(trigger),
                    target: this.target(trigger),
                    text: this.text(trigger),
                    container: this.container,
                    trigger: trigger,
                    emitter: this
                });
            }
        }, {
            key: 'defaultAction',
            value: function defaultAction(trigger) {
                return getAttributeValue('action', trigger);
            }
        }, {
            key: 'defaultTarget',
            value: function defaultTarget(trigger) {
                var selector = getAttributeValue('target', trigger);

                if (selector) {
                    return document.querySelector(selector);
                }
            }
        }, {
            key: 'defaultText',
            value: function defaultText(trigger) {
                return getAttributeValue('text', trigger);
            }
        }, {
            key: 'destroy',
            value: function destroy() {
                this.listener.destroy();

                if (this.clipboardAction) {
                    this.clipboardAction.destroy();
                    this.clipboardAction = null;
                }
            }
        }], [{
            key: 'isSupported',
            value: function isSupported() {
                var action = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : ['copy', 'cut'];

                var actions = typeof action === 'string' ? [action] : action;
                var support = !!document.queryCommandSupported;

                actions.forEach(function (action) {
                    support = support && !!document.queryCommandSupported(action);
                });

                return support;
            }
        }]);

        return Clipboard;
    }(_tinyEmitter2.default);

    /**
     * Helper function to retrieve attribute value.
     * @param {String} suffix
     * @param {Element} element
     */
    function getAttributeValue(suffix, element) {
        var attribute = 'data-clipboard-' + suffix;

        if (!element.hasAttribute(attribute)) {
            return;
        }

        return element.getAttribute(attribute);
    }

    module.exports = Clipboard;
});

},{"./clipboard-action":7,"good-listener":4,"tiny-emitter":6}]},{},[8])(8)
});;/*! moip-2.7.1.js - build date: 2016-08-30 */
var JSEncryptExports={};!function(a){function b(a,b,c){null!=a&&("number"==typeof a?this.fromNumber(a,b,c):null==b&&"string"!=typeof a?this.fromString(a,256):this.fromString(a,b))}function c(){return new b(null)}function d(a,b,c,d,e,f){for(;0<=--f;){var g=b*this[a++]+c[d]+e;e=Math.floor(g/67108864),c[d++]=67108863&g}return e}function e(a,b,c,d,e,f){var g=32767&b;for(b>>=15;0<=--f;){var h=32767&this[a],i=this[a++]>>15,j=b*h+i*g,h=g*h+((32767&j)<<15)+c[d]+(1073741823&e);e=(h>>>30)+(j>>>15)+b*i+(e>>>30),c[d++]=1073741823&h}return e}function f(a,b,c,d,e,f){var g=16383&b;for(b>>=14;0<=--f;){var h=16383&this[a],i=this[a++]>>14,j=b*h+i*g,h=g*h+((16383&j)<<14)+c[d]+e;e=(h>>28)+(j>>14)+b*i,c[d++]=268435455&h}return e}function g(a,b){var c=C[a.charCodeAt(b)];return null==c?-1:c}function h(a){var b=c();return b.fromInt(a),b}function i(a){var b,c=1;return 0!=(b=a>>>16)&&(a=b,c+=16),0!=(b=a>>8)&&(a=b,c+=8),0!=(b=a>>4)&&(a=b,c+=4),0!=(b=a>>2)&&(a=b,c+=2),0!=a>>1&&(c+=1),c}function j(a){this.m=a}function k(a){this.m=a,this.mp=a.invDigit(),this.mpl=32767&this.mp,this.mph=this.mp>>15,this.um=(1<<a.DB-15)-1,this.mt2=2*a.t}function l(a,b){return a&b}function m(a,b){return a|b}function n(a,b){return a^b}function o(a,b){return a&~b}function p(){}function q(a){return a}function r(a){this.r2=c(),this.q3=c(),b.ONE.dlShiftTo(2*a.t,this.r2),this.mu=this.r2.divide(a),this.m=a}function s(){this.j=this.i=0,this.S=[]}function t(){}function u(a,c){return new b(a,c)}function w(){this.n=null,this.e=0,this.coeff=this.dmq1=this.dmp1=this.q=this.p=this.d=null}function x(a){var b,c,d="";for(b=0;b+3<=a.length;b+=3)c=parseInt(a.substring(b,b+3),16),d+=J.charAt(c>>6)+J.charAt(63&c);for(b+1==a.length?(c=parseInt(a.substring(b,b+1),16),d+=J.charAt(c<<2)):b+2==a.length&&(c=parseInt(a.substring(b,b+2),16),d+=J.charAt(c>>2)+J.charAt((3&c)<<4));0<(3&d.length);)d+=K;return d}function y(a){var b,c,d="",e=0;for(b=0;b<a.length&&a.charAt(b)!=K;++b)v=J.indexOf(a.charAt(b)),0>v||(0==e?(d+=B.charAt(v>>2),c=3&v,e=1):1==e?(d+=B.charAt(c<<2|v>>4),c=15&v,e=2):2==e?(d+=B.charAt(c),d+=B.charAt(v>>2),c=3&v,e=3):(d+=B.charAt(c<<2|v>>4),d+=B.charAt(15&v),e=0));return 1==e&&(d+=B.charAt(c<<2)),d}var z;"Microsoft Internet Explorer"==navigator.appName?(b.prototype.am=e,z=30):"Netscape"!=navigator.appName?(b.prototype.am=d,z=26):(b.prototype.am=f,z=28),b.prototype.DB=z,b.prototype.DM=(1<<z)-1,b.prototype.DV=1<<z,b.prototype.FV=Math.pow(2,52),b.prototype.F1=52-z,b.prototype.F2=2*z-52;var A,B="0123456789abcdefghijklmnopqrstuvwxyz",C=[];for(z=48,A=0;9>=A;++A)C[z++]=A;for(z=97,A=10;36>A;++A)C[z++]=A;for(z=65,A=10;36>A;++A)C[z++]=A;j.prototype.convert=function(a){return 0>a.s||0<=a.compareTo(this.m)?a.mod(this.m):a},j.prototype.revert=function(a){return a},j.prototype.reduce=function(a){a.divRemTo(this.m,null,a)},j.prototype.mulTo=function(a,b,c){a.multiplyTo(b,c),this.reduce(c)},j.prototype.sqrTo=function(a,b){a.squareTo(b),this.reduce(b)},k.prototype.convert=function(a){var d=c();return a.abs().dlShiftTo(this.m.t,d),d.divRemTo(this.m,null,d),0>a.s&&0<d.compareTo(b.ZERO)&&this.m.subTo(d,d),d},k.prototype.revert=function(a){var b=c();return a.copyTo(b),this.reduce(b),b},k.prototype.reduce=function(a){for(;a.t<=this.mt2;)a[a.t++]=0;for(var b=0;b<this.m.t;++b){var c=32767&a[b],d=c*this.mpl+((c*this.mph+(a[b]>>15)*this.mpl&this.um)<<15)&a.DM,c=b+this.m.t;for(a[c]+=this.m.am(0,d,a,b,0,this.m.t);a[c]>=a.DV;)a[c]-=a.DV,a[++c]++}a.clamp(),a.drShiftTo(this.m.t,a),0<=a.compareTo(this.m)&&a.subTo(this.m,a)},k.prototype.mulTo=function(a,b,c){a.multiplyTo(b,c),this.reduce(c)},k.prototype.sqrTo=function(a,b){a.squareTo(b),this.reduce(b)},b.prototype.copyTo=function(a){for(var b=this.t-1;b>=0;--b)a[b]=this[b];a.t=this.t,a.s=this.s},b.prototype.fromInt=function(a){this.t=1,this.s=0>a?-1:0,a>0?this[0]=a:-1>a?this[0]=a+DV:this.t=0},b.prototype.fromString=function(a,c){var d;if(16==c)d=4;else if(8==c)d=3;else if(256==c)d=8;else if(2==c)d=1;else if(32==c)d=5;else{if(4!=c)return void this.fromRadix(a,c);d=2}this.s=this.t=0;for(var e=a.length,f=!1,h=0;0<=--e;){var i=8==d?255&a[e]:g(a,e);0>i?"-"==a.charAt(e)&&(f=!0):(f=!1,0==h?this[this.t++]=i:h+d>this.DB?(this[this.t-1]|=(i&(1<<this.DB-h)-1)<<h,this[this.t++]=i>>this.DB-h):this[this.t-1]|=i<<h,h+=d,h>=this.DB&&(h-=this.DB))}8==d&&0!=(128&a[0])&&(this.s=-1,h>0&&(this[this.t-1]|=(1<<this.DB-h)-1<<h)),this.clamp(),f&&b.ZERO.subTo(this,this)},b.prototype.clamp=function(){for(var a=this.s&this.DM;0<this.t&&this[this.t-1]==a;)--this.t},b.prototype.dlShiftTo=function(a,b){var c;for(c=this.t-1;c>=0;--c)b[c+a]=this[c];for(c=a-1;c>=0;--c)b[c]=0;b.t=this.t+a,b.s=this.s},b.prototype.drShiftTo=function(a,b){for(var c=a;c<this.t;++c)b[c-a]=this[c];b.t=Math.max(this.t-a,0),b.s=this.s},b.prototype.lShiftTo=function(a,b){var c,d=a%this.DB,e=this.DB-d,f=(1<<e)-1,g=Math.floor(a/this.DB),h=this.s<<d&this.DM;for(c=this.t-1;c>=0;--c)b[c+g+1]=this[c]>>e|h,h=(this[c]&f)<<d;for(c=g-1;c>=0;--c)b[c]=0;b[g]=h,b.t=this.t+g+1,b.s=this.s,b.clamp()},b.prototype.rShiftTo=function(a,b){b.s=this.s;var c=Math.floor(a/this.DB);if(c>=this.t)b.t=0;else{var d=a%this.DB,e=this.DB-d,f=(1<<d)-1;b[0]=this[c]>>d;for(var g=c+1;g<this.t;++g)b[g-c-1]|=(this[g]&f)<<e,b[g-c]=this[g]>>d;d>0&&(b[this.t-c-1]|=(this.s&f)<<e),b.t=this.t-c,b.clamp()}},b.prototype.subTo=function(a,b){for(var c=0,d=0,e=Math.min(a.t,this.t);e>c;)d+=this[c]-a[c],b[c++]=d&this.DM,d>>=this.DB;if(a.t<this.t){for(d-=a.s;c<this.t;)d+=this[c],b[c++]=d&this.DM,d>>=this.DB;d+=this.s}else{for(d+=this.s;c<a.t;)d-=a[c],b[c++]=d&this.DM,d>>=this.DB;d-=a.s}b.s=0>d?-1:0,-1>d?b[c++]=this.DV+d:d>0&&(b[c++]=d),b.t=c,b.clamp()},b.prototype.multiplyTo=function(a,c){var d=this.abs(),e=a.abs(),f=d.t;for(c.t=f+e.t;0<=--f;)c[f]=0;for(f=0;f<e.t;++f)c[f+d.t]=d.am(0,e[f],c,f,0,d.t);c.s=0,c.clamp(),this.s!=a.s&&b.ZERO.subTo(c,c)},b.prototype.squareTo=function(a){for(var b=this.abs(),c=a.t=2*b.t;0<=--c;)a[c]=0;for(c=0;c<b.t-1;++c){var d=b.am(c,b[c],a,2*c,0,1);(a[c+b.t]+=b.am(c+1,2*b[c],a,2*c+1,d,b.t-c-1))>=b.DV&&(a[c+b.t]-=b.DV,a[c+b.t+1]=1)}0<a.t&&(a[a.t-1]+=b.am(c,b[c],a,2*c,0,1)),a.s=0,a.clamp()},b.prototype.divRemTo=function(a,d,e){var f=a.abs();if(!(0>=f.t)){var g=this.abs();if(g.t<f.t)null!=d&&d.fromInt(0),null!=e&&this.copyTo(e);else{null==e&&(e=c());var h=c(),j=this.s;a=a.s;var k=this.DB-i(f[f.t-1]);if(k>0?(f.lShiftTo(k,h),g.lShiftTo(k,e)):(f.copyTo(h),g.copyTo(e)),f=h.t,g=h[f-1],0!=g){var l=g*(1<<this.F1)+(f>1?h[f-2]>>this.F2:0),m=this.FV/l,l=(1<<this.F1)/l,n=1<<this.F2,o=e.t,p=o-f,q=null==d?c():d;for(h.dlShiftTo(p,q),0<=e.compareTo(q)&&(e[e.t++]=1,e.subTo(q,e)),b.ONE.dlShiftTo(f,q),q.subTo(h,h);h.t<f;)h[h.t++]=0;for(;0<=--p;){var r=e[--o]==g?this.DM:Math.floor(e[o]*m+(e[o-1]+n)*l);if((e[o]+=h.am(0,r,e,p,0,f))<r)for(h.dlShiftTo(p,q),e.subTo(q,e);e[o]<--r;)e.subTo(q,e)}null!=d&&(e.drShiftTo(f,d),j!=a&&b.ZERO.subTo(d,d)),e.t=f,e.clamp(),k>0&&e.rShiftTo(k,e),0>j&&b.ZERO.subTo(e,e)}}}},b.prototype.invDigit=function(){if(1>this.t)return 0;var a=this[0];if(0==(1&a))return 0;var b=3&a,b=b*(2-(15&a)*b)&15,b=b*(2-(255&a)*b)&255,b=b*(2-((65535&a)*b&65535))&65535,b=b*(2-a*b%this.DV)%this.DV;return b>0?this.DV-b:-b},b.prototype.isEven=function(){return 0==(0<this.t?1&this[0]:this.s)},b.prototype.exp=function(a,d){if(a>4294967295||1>a)return b.ONE;var e=c(),f=c(),g=d.convert(this),h=i(a)-1;for(g.copyTo(e);0<=--h;)if(d.sqrTo(e,f),(a&1<<h)>0)d.mulTo(f,g,e);else var j=e,e=f,f=j;return d.revert(e)},b.prototype.toString=function(a){if(0>this.s)return"-"+this.negate().toString(a);if(16==a)a=4;else if(8==a)a=3;else if(2==a)a=1;else if(32==a)a=5;else{if(4!=a)return this.toRadix(a);a=2}var b,c=(1<<a)-1,d=!1,e="",f=this.t,g=this.DB-f*this.DB%a;if(0<f--)for(g<this.DB&&0<(b=this[f]>>g)&&(d=!0,e=B.charAt(b));f>=0;)a>g?(b=(this[f]&(1<<g)-1)<<a-g,b|=this[--f]>>(g+=this.DB-a)):(b=this[f]>>(g-=a)&c,0>=g&&(g+=this.DB,--f)),b>0&&(d=!0),d&&(e+=B.charAt(b));return d?e:"0"},b.prototype.negate=function(){var a=c();return b.ZERO.subTo(this,a),a},b.prototype.abs=function(){return 0>this.s?this.negate():this},b.prototype.compareTo=function(a){var b=this.s-a.s;if(0!=b)return b;var c=this.t,b=c-a.t;if(0!=b)return 0>this.s?-b:b;for(;0<=--c;)if(0!=(b=this[c]-a[c]))return b;return 0},b.prototype.bitLength=function(){return 0>=this.t?0:this.DB*(this.t-1)+i(this[this.t-1]^this.s&this.DM)},b.prototype.mod=function(a){var d=c();return this.abs().divRemTo(a,null,d),0>this.s&&0<d.compareTo(b.ZERO)&&a.subTo(d,d),d},b.prototype.modPowInt=function(a,b){var c;return c=256>a||b.isEven()?new j(b):new k(b),this.exp(a,c)},b.ZERO=h(0),b.ONE=h(1),p.prototype.convert=q,p.prototype.revert=q,p.prototype.mulTo=function(a,b,c){a.multiplyTo(b,c)},p.prototype.sqrTo=function(a,b){a.squareTo(b)},r.prototype.convert=function(a){if(0>a.s||a.t>2*this.m.t)return a.mod(this.m);if(0>a.compareTo(this.m))return a;var b=c();return a.copyTo(b),this.reduce(b),b},r.prototype.revert=function(a){return a},r.prototype.reduce=function(a){for(a.drShiftTo(this.m.t-1,this.r2),a.t>this.m.t+1&&(a.t=this.m.t+1,a.clamp()),this.mu.multiplyUpperTo(this.r2,this.m.t+1,this.q3),this.m.multiplyLowerTo(this.q3,this.m.t+1,this.r2);0>a.compareTo(this.r2);)a.dAddOffset(1,this.m.t+1);for(a.subTo(this.r2,a);0<=a.compareTo(this.m);)a.subTo(this.m,a)},r.prototype.mulTo=function(a,b,c){a.multiplyTo(b,c),this.reduce(c)},r.prototype.sqrTo=function(a,b){a.squareTo(b),this.reduce(b)};var D=[2,3,5,7,11,13,17,19,23,29,31,37,41,43,47,53,59,61,67,71,73,79,83,89,97,101,103,107,109,113,127,131,137,139,149,151,157,163,167,173,179,181,191,193,197,199,211,223,227,229,233,239,241,251,257,263,269,271,277,281,283,293,307,311,313,317,331,337,347,349,353,359,367,373,379,383,389,397,401,409,419,421,431,433,439,443,449,457,461,463,467,479,487,491,499,503,509,521,523,541,547,557,563,569,571,577,587,593,599,601,607,613,617,619,631,641,643,647,653,659,661,673,677,683,691,701,709,719,727,733,739,743,751,757,761,769,773,787,797,809,811,821,823,827,829,839,853,857,859,863,877,881,883,887,907,911,919,929,937,941,947,953,967,971,977,983,991,997],E=67108864/D[D.length-1];b.prototype.chunkSize=function(a){return Math.floor(Math.LN2*this.DB/Math.log(a))},b.prototype.toRadix=function(a){if(null==a&&(a=10),0==this.signum()||2>a||a>36)return"0";var b=this.chunkSize(a),b=Math.pow(a,b),d=h(b),e=c(),f=c(),g="";for(this.divRemTo(d,e,f);0<e.signum();)g=(b+f.intValue()).toString(a).substr(1)+g,e.divRemTo(d,e,f);return f.intValue().toString(a)+g},b.prototype.fromRadix=function(a,c){this.fromInt(0),null==c&&(c=10);for(var d=this.chunkSize(c),e=Math.pow(c,d),f=!1,h=0,i=0,j=0;j<a.length;++j){var k=g(a,j);0>k?"-"==a.charAt(j)&&0==this.signum()&&(f=!0):(i=c*i+k,++h>=d&&(this.dMultiply(e),this.dAddOffset(i,0),i=h=0))}h>0&&(this.dMultiply(Math.pow(c,h)),this.dAddOffset(i,0)),f&&b.ZERO.subTo(this,this)},b.prototype.fromNumber=function(a,c,d){if("number"==typeof c)if(2>a)this.fromInt(1);else for(this.fromNumber(a,d),this.testBit(a-1)||this.bitwiseTo(b.ONE.shiftLeft(a-1),m,this),this.isEven()&&this.dAddOffset(1,0);!this.isProbablePrime(c);)this.dAddOffset(2,0),this.bitLength()>a&&this.subTo(b.ONE.shiftLeft(a-1),this);else{d=[];var e=7&a;d.length=(a>>3)+1,c.nextBytes(d),d[0]=e>0?d[0]&(1<<e)-1:0,this.fromString(d,256)}},b.prototype.bitwiseTo=function(a,b,c){var d,e,f=Math.min(a.t,this.t);for(d=0;f>d;++d)c[d]=b(this[d],a[d]);if(a.t<this.t){for(e=a.s&this.DM,d=f;d<this.t;++d)c[d]=b(this[d],e);c.t=this.t}else{for(e=this.s&this.DM,d=f;d<a.t;++d)c[d]=b(e,a[d]);c.t=a.t}c.s=b(this.s,a.s),c.clamp()},b.prototype.changeBit=function(a,c){var d=b.ONE.shiftLeft(a);return this.bitwiseTo(d,c,d),d},b.prototype.addTo=function(a,b){for(var c=0,d=0,e=Math.min(a.t,this.t);e>c;)d+=this[c]+a[c],b[c++]=d&this.DM,d>>=this.DB;if(a.t<this.t){for(d+=a.s;c<this.t;)d+=this[c],b[c++]=d&this.DM,d>>=this.DB;d+=this.s}else{for(d+=this.s;c<a.t;)d+=a[c],b[c++]=d&this.DM,d>>=this.DB;d+=a.s}b.s=0>d?-1:0,d>0?b[c++]=d:-1>d&&(b[c++]=this.DV+d),b.t=c,b.clamp()},b.prototype.dMultiply=function(a){this[this.t]=this.am(0,a-1,this,0,0,this.t),++this.t,this.clamp()},b.prototype.dAddOffset=function(a,b){if(0!=a){for(;this.t<=b;)this[this.t++]=0;for(this[b]+=a;this[b]>=this.DV;)this[b]-=this.DV,++b>=this.t&&(this[this.t++]=0),++this[b]}},b.prototype.multiplyLowerTo=function(a,b,c){var d=Math.min(this.t+a.t,b);for(c.s=0,c.t=d;d>0;)c[--d]=0;var e;for(e=c.t-this.t;e>d;++d)c[d+this.t]=this.am(0,a[d],c,d,0,this.t);for(e=Math.min(a.t,b);e>d;++d)this.am(0,a[d],c,d,0,b-d);c.clamp()},b.prototype.multiplyUpperTo=function(a,b,c){--b;var d=c.t=this.t+a.t-b;for(c.s=0;0<=--d;)c[d]=0;for(d=Math.max(b-this.t,0);d<a.t;++d)c[this.t+d-b]=this.am(b-d,a[d],c,0,0,this.t+d-b);c.clamp(),c.drShiftTo(1,c)},b.prototype.modInt=function(a){if(0>=a)return 0;var b=this.DV%a,c=0>this.s?a-1:0;if(0<this.t)if(0==b)c=this[0]%a;else for(var d=this.t-1;d>=0;--d)c=(b*c+this[d])%a;return c},b.prototype.millerRabin=function(a){var d=this.subtract(b.ONE),e=d.getLowestSetBit();if(0>=e)return!1;var f=d.shiftRight(e);a=a+1>>1,a>D.length&&(a=D.length);for(var g=c(),h=0;a>h;++h){g.fromInt(D[Math.floor(Math.random()*D.length)]);var i=g.modPow(f,this);if(0!=i.compareTo(b.ONE)&&0!=i.compareTo(d)){for(var j=1;j++<e&&0!=i.compareTo(d);)if(i=i.modPowInt(2,this),0==i.compareTo(b.ONE))return!1;if(0!=i.compareTo(d))return!1}}return!0},b.prototype.clone=function(){var a=c();return this.copyTo(a),a},b.prototype.intValue=function(){if(0>this.s){if(1==this.t)return this[0]-this.DV;if(0==this.t)return-1}else{if(1==this.t)return this[0];if(0==this.t)return 0}return(this[1]&(1<<32-this.DB)-1)<<this.DB|this[0]},b.prototype.byteValue=function(){return 0==this.t?this.s:this[0]<<24>>24},b.prototype.shortValue=function(){return 0==this.t?this.s:this[0]<<16>>16},b.prototype.signum=function(){return 0>this.s?-1:0>=this.t||1==this.t&&0>=this[0]?0:1},b.prototype.toByteArray=function(){var a=this.t,b=[];b[0]=this.s;var c,d=this.DB-a*this.DB%8,e=0;if(0<a--)for(d<this.DB&&(c=this[a]>>d)!=(this.s&this.DM)>>d&&(b[e++]=c|this.s<<this.DB-d);a>=0;)8>d?(c=(this[a]&(1<<d)-1)<<8-d,c|=this[--a]>>(d+=this.DB-8)):(c=this[a]>>(d-=8)&255,0>=d&&(d+=this.DB,--a)),0!=(128&c)&&(c|=-256),0==e&&(128&this.s)!=(128&c)&&++e,(e>0||c!=this.s)&&(b[e++]=c);return b},b.prototype.equals=function(a){return 0==this.compareTo(a)},b.prototype.min=function(a){return 0>this.compareTo(a)?this:a},b.prototype.max=function(a){return 0<this.compareTo(a)?this:a},b.prototype.and=function(a){var b=c();return this.bitwiseTo(a,l,b),b},b.prototype.or=function(a){var b=c();return this.bitwiseTo(a,m,b),b},b.prototype.xor=function(a){var b=c();return this.bitwiseTo(a,n,b),b},b.prototype.andNot=function(a){var b=c();return this.bitwiseTo(a,o,b),b},b.prototype.not=function(){for(var a=c(),b=0;b<this.t;++b)a[b]=this.DM&~this[b];return a.t=this.t,a.s=~this.s,a},b.prototype.shiftLeft=function(a){var b=c();return 0>a?this.rShiftTo(-a,b):this.lShiftTo(a,b),b},b.prototype.shiftRight=function(a){var b=c();return 0>a?this.lShiftTo(-a,b):this.rShiftTo(a,b),b},b.prototype.getLowestSetBit=function(){for(var a=0;a<this.t;++a)if(0!=this[a]){var b=a*this.DB;if(a=this[a],0==a)a=-1;else{var c=0;0==(65535&a)&&(a>>=16,c+=16),0==(255&a)&&(a>>=8,c+=8),0==(15&a)&&(a>>=4,c+=4),0==(3&a)&&(a>>=2,c+=2),0==(1&a)&&++c,a=c}return b+a}return 0>this.s?this.t*this.DB:-1},b.prototype.bitCount=function(){for(var a=0,b=this.s&this.DM,c=0;c<this.t;++c){for(var d=this[c]^b,e=0;0!=d;)d&=d-1,++e;a+=e}return a},b.prototype.testBit=function(a){var b=Math.floor(a/this.DB);return b>=this.t?0!=this.s:0!=(this[b]&1<<a%this.DB)},b.prototype.setBit=function(a){return this.changeBit(a,m)},b.prototype.clearBit=function(a){return this.changeBit(a,o)},b.prototype.flipBit=function(a){return this.changeBit(a,n)},b.prototype.add=function(a){var b=c();return this.addTo(a,b),b},b.prototype.subtract=function(a){var b=c();return this.subTo(a,b),b},b.prototype.multiply=function(a){var b=c();return this.multiplyTo(a,b),b},b.prototype.divide=function(a){var b=c();return this.divRemTo(a,b,null),b},b.prototype.remainder=function(a){var b=c();return this.divRemTo(a,null,b),b},b.prototype.divideAndRemainder=function(a){var b=c(),d=c();return this.divRemTo(a,b,d),[b,d]},b.prototype.modPow=function(a,b){var d,e,f=a.bitLength(),g=h(1);if(0>=f)return g;d=18>f?1:48>f?3:144>f?4:768>f?5:6,e=8>f?new j(b):b.isEven()?new r(b):new k(b);var l=[],m=3,n=d-1,o=(1<<d)-1;if(l[1]=e.convert(this),d>1)for(f=c(),e.sqrTo(l[1],f);o>=m;)l[m]=c(),e.mulTo(f,l[m-2],l[m]),m+=2;for(var p,q=a.t-1,s=!0,t=c(),f=i(a[q])-1;q>=0;){for(f>=n?p=a[q]>>f-n&o:(p=(a[q]&(1<<f+1)-1)<<n-f,q>0&&(p|=a[q-1]>>this.DB+f-n)),m=d;0==(1&p);)p>>=1,--m;if(0>(f-=m)&&(f+=this.DB,--q),s)l[p].copyTo(g),s=!1;else{for(;m>1;)e.sqrTo(g,t),e.sqrTo(t,g),m-=2;m>0?e.sqrTo(g,t):(m=g,g=t,t=m),e.mulTo(t,l[p],g)}for(;q>=0&&0==(a[q]&1<<f);)e.sqrTo(g,t),m=g,g=t,t=m,0>--f&&(f=this.DB-1,--q)}return e.revert(g)},b.prototype.modInverse=function(a){var c=a.isEven();if(this.isEven()&&c||0==a.signum())return b.ZERO;for(var d=a.clone(),e=this.clone(),f=h(1),g=h(0),i=h(0),j=h(1);0!=d.signum();){for(;d.isEven();)d.rShiftTo(1,d),c?(f.isEven()&&g.isEven()||(f.addTo(this,f),g.subTo(a,g)),f.rShiftTo(1,f)):g.isEven()||g.subTo(a,g),g.rShiftTo(1,g);for(;e.isEven();)e.rShiftTo(1,e),c?(i.isEven()&&j.isEven()||(i.addTo(this,i),j.subTo(a,j)),i.rShiftTo(1,i)):j.isEven()||j.subTo(a,j),j.rShiftTo(1,j);0<=d.compareTo(e)?(d.subTo(e,d),c&&f.subTo(i,f),g.subTo(j,g)):(e.subTo(d,e),c&&i.subTo(f,i),j.subTo(g,j))}return 0!=e.compareTo(b.ONE)?b.ZERO:0<=j.compareTo(a)?j.subtract(a):0>j.signum()?(j.addTo(a,j),0>j.signum()?j.add(a):j):j},b.prototype.pow=function(a){return this.exp(a,new p)},b.prototype.gcd=function(a){var b=0>this.s?this.negate():this.clone();if(a=0>a.s?a.negate():a.clone(),0>b.compareTo(a)){var c=b,b=a;a=c}var c=b.getLowestSetBit(),d=a.getLowestSetBit();if(0>d)return b;for(d>c&&(d=c),d>0&&(b.rShiftTo(d,b),a.rShiftTo(d,a));0<b.signum();)0<(c=b.getLowestSetBit())&&b.rShiftTo(c,b),0<(c=a.getLowestSetBit())&&a.rShiftTo(c,a),0<=b.compareTo(a)?(b.subTo(a,b),b.rShiftTo(1,b)):(a.subTo(b,a),a.rShiftTo(1,a));return d>0&&a.lShiftTo(d,a),a},b.prototype.isProbablePrime=function(a){var b,c=this.abs();if(1==c.t&&c[0]<=D[D.length-1]){for(b=0;b<D.length;++b)if(c[0]==D[b])return!0;return!1}if(c.isEven())return!1;for(b=1;b<D.length;){for(var d=D[b],e=b+1;e<D.length&&E>d;)d*=D[e++];for(d=c.modInt(d);e>b;)if(0==d%D[b++])return!1}return c.millerRabin(a)},b.prototype.square=function(){var a=c();return this.squareTo(a),a},s.prototype.init=function(a){var b,c,d;for(b=0;256>b;++b)this.S[b]=b;for(b=c=0;256>b;++b)c=c+this.S[b]+a[b%a.length]&255,d=this.S[b],this.S[b]=this.S[c],this.S[c]=d;this.j=this.i=0},s.prototype.next=function(){var a;return this.i=this.i+1&255,this.j=this.j+this.S[this.i]&255,a=this.S[this.i],this.S[this.i]=this.S[this.j],this.S[this.j]=a,this.S[a+this.S[this.i]&255]};var F,G,H;if(null==G){if(G=[],H=0,window.crypto&&window.crypto.getRandomValues)for(A=new Uint32Array(256),window.crypto.getRandomValues(A),z=0;z<A.length;++z)G[H++]=255&A[z];var I=function(a){this.count=this.count||0,256<=this.count||H>=256?window.removeEventListener?window.removeEventListener("mousemove",I):window.detachEvent&&window.detachEvent("onmousemove",I):(this.count+=1,a=a.x+a.y,G[H++]=255&a)};window.addEventListener?window.addEventListener("mousemove",I):window.attachEvent&&window.attachEvent("onmousemove",I)}t.prototype.nextBytes=function(a){var b;for(b=0;b<a.length;++b){var c,d=a,e=b;if(null==F){for(F=new s;256>H;)c=Math.floor(65536*Math.random()),G[H++]=255&c;for(F.init(G),H=0;H<G.length;++H)G[H]=0;H=0}c=F.next(),d[e]=c}},w.prototype.doPublic=function(a){return a.modPowInt(this.e,this.n)},w.prototype.setPublic=function(a,b){null!=a&&null!=b&&0<a.length&&0<b.length?(this.n=u(a,16),this.e=parseInt(b,16)):console.error("Invalid RSA public key")},w.prototype.encrypt=function(a){var c;if(c=this.n.bitLength()+7>>3,c<a.length+11)console.error("Message too long for RSA"),c=null;else{for(var d=[],e=a.length-1;e>=0&&c>0;){var f=a.charCodeAt(e--);128>f?d[--c]=f:f>127&&2048>f?(d[--c]=63&f|128,d[--c]=f>>6|192):(d[--c]=63&f|128,d[--c]=f>>6&63|128,d[--c]=f>>12|224)}for(d[--c]=0,a=new t,e=[];c>2;){for(e[0]=0;0==e[0];)a.nextBytes(e);d[--c]=e[0]}d[--c]=2,d[--c]=0,c=new b(d)}return null==c?null:(c=this.doPublic(c),null==c?null:(c=c.toString(16),0==(1&c.length)?c:"0"+c))},w.prototype.doPrivate=function(a){if(null==this.p||null==this.q)return a.modPow(this.d,this.n);var b=a.mod(this.p).modPow(this.dmp1,this.p);for(a=a.mod(this.q).modPow(this.dmq1,this.q);0>b.compareTo(a);)b=b.add(this.p);return b.subtract(a).multiply(this.coeff).mod(this.p).multiply(this.q).add(a)},w.prototype.setPrivate=function(a,b,c){null!=a&&null!=b&&0<a.length&&0<b.length?(this.n=u(a,16),this.e=parseInt(b,16),this.d=u(c,16)):console.error("Invalid RSA private key")},w.prototype.setPrivateEx=function(a,b,c,d,e,f,g,h){null!=a&&null!=b&&0<a.length&&0<b.length?(this.n=u(a,16),this.e=parseInt(b,16),this.d=u(c,16),this.p=u(d,16),this.q=u(e,16),this.dmp1=u(f,16),this.dmq1=u(g,16),this.coeff=u(h,16)):console.error("Invalid RSA private key")},w.prototype.generate=function(a,c){var d=new t,e=a>>1;this.e=parseInt(c,16);for(var f=new b(c,16);;){for(;this.p=new b(a-e,1,d),0!=this.p.subtract(b.ONE).gcd(f).compareTo(b.ONE)||!this.p.isProbablePrime(10););for(;this.q=new b(e,1,d),0!=this.q.subtract(b.ONE).gcd(f).compareTo(b.ONE)||!this.q.isProbablePrime(10););if(0>=this.p.compareTo(this.q)){var g=this.p;this.p=this.q,this.q=g}var g=this.p.subtract(b.ONE),h=this.q.subtract(b.ONE),i=g.multiply(h);if(0==i.gcd(f).compareTo(b.ONE)){this.n=this.p.multiply(this.q),this.d=f.modInverse(i),this.dmp1=this.d.mod(g),this.dmq1=this.d.mod(h),this.coeff=this.q.modInverse(this.p);break}}},w.prototype.decrypt=function(a){if(a=u(a,16),a=this.doPrivate(a),null==a)return null;a:{var b=this.n.bitLength()+7>>3;a=a.toByteArray();for(var c=0;c<a.length&&0==a[c];)++c;if(a.length-c!=b-1||2!=a[c])a=null;else{for(++c;0!=a[c];)if(++c>=a.length){a=null;break a}for(b="";++c<a.length;){var d=255&a[c];128>d?b+=String.fromCharCode(d):d>191&&224>d?(b+=String.fromCharCode((31&d)<<6|63&a[c+1]),++c):(b+=String.fromCharCode((15&d)<<12|(63&a[c+1])<<6|63&a[c+2]),c+=2)}a=b}}return a},function(){w.prototype.generateAsync=function(a,d,e){var f=new t,g=a>>1;this.e=parseInt(d,16);var h=new b(d,16),i=this,j=function(){var d=function(){if(0>=i.p.compareTo(i.q)){var a=i.p;i.p=i.q,i.q=a}var a=i.p.subtract(b.ONE),c=i.q.subtract(b.ONE),d=a.multiply(c);0==d.gcd(h).compareTo(b.ONE)?(i.n=i.p.multiply(i.q),i.d=h.modInverse(d),i.dmp1=i.d.mod(a),i.dmq1=i.d.mod(c),i.coeff=i.q.modInverse(i.p),setTimeout(function(){e()},0)):setTimeout(j,0)},k=function(){i.q=c(),i.q.fromNumberAsync(g,1,f,function(){i.q.subtract(b.ONE).gcda(h,function(a){0==a.compareTo(b.ONE)&&i.q.isProbablePrime(10)?setTimeout(d,0):setTimeout(k,0)})})},l=function(){i.p=c(),i.p.fromNumberAsync(a-g,1,f,function(){i.p.subtract(b.ONE).gcda(h,function(a){0==a.compareTo(b.ONE)&&i.p.isProbablePrime(10)?setTimeout(k,0):setTimeout(l,0)})})};setTimeout(l,0)};setTimeout(j,0)},b.prototype.gcda=function(a,b){var c=0>this.s?this.negate():this.clone(),d=0>a.s?a.negate():a.clone();if(0>c.compareTo(d))var e=c,c=d,d=e;var f=c.getLowestSetBit(),g=d.getLowestSetBit();if(0>g)b(c);else{g>f&&(g=f),g>0&&(c.rShiftTo(g,c),d.rShiftTo(g,d));var h=function(){0<(f=c.getLowestSetBit())&&c.rShiftTo(f,c),0<(f=d.getLowestSetBit())&&d.rShiftTo(f,d),0<=c.compareTo(d)?(c.subTo(d,c),c.rShiftTo(1,c)):(d.subTo(c,d),d.rShiftTo(1,d)),0<c.signum()?setTimeout(h,0):(g>0&&d.lShiftTo(g,d),setTimeout(function(){b(d)},0))};setTimeout(h,10)}},b.prototype.fromNumberAsync=function(a,c,d,e){if("number"==typeof c)if(2>a)this.fromInt(1);else{this.fromNumber(a,d),this.testBit(a-1)||this.bitwiseTo(b.ONE.shiftLeft(a-1),m,this),this.isEven()&&this.dAddOffset(1,0);var f=this,g=function(){f.dAddOffset(2,0),f.bitLength()>a&&f.subTo(b.ONE.shiftLeft(a-1),f),f.isProbablePrime(c)?setTimeout(function(){e()},0):setTimeout(g,0)};setTimeout(g,0)}else{d=[];var h=7&a;d.length=(a>>3)+1,c.nextBytes(d),d[0]=h>0?d[0]&(1<<h)-1:0,this.fromString(d,256)}}}();var J="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",K="=",L=L||{};L.env=L.env||{};var M=L,N=Object.prototype,O=["toString","valueOf"];L.env.parseUA=function(a){var b=function(a){var b=0;return parseFloat(a.replace(/\./g,function(){return 1==b++?"":"."}))},c=navigator,c={ie:0,opera:0,gecko:0,webkit:0,chrome:0,mobile:null,air:0,ipad:0,iphone:0,ipod:0,ios:null,android:0,webos:0,caja:c&&c.cajaVersion,secure:!1,os:null};a=a||navigator&&navigator.userAgent;var d=window&&window.location,d=d&&d.href;return c.secure=d&&0===d.toLowerCase().indexOf("https"),a&&(/windows|win32/i.test(a)?c.os="windows":/macintosh/i.test(a)?c.os="macintosh":/rhino/i.test(a)&&(c.os="rhino"),/KHTML/.test(a)&&(c.webkit=1),(d=a.match(/AppleWebKit\/([^\s]*)/))&&d[1]&&(c.webkit=b(d[1]),/ Mobile\//.test(a)?(c.mobile="Apple",(d=a.match(/OS ([^\s]*)/))&&d[1]&&(d=b(d[1].replace("_","."))),c.ios=d,c.ipad=c.ipod=c.iphone=0,(d=a.match(/iPad|iPod|iPhone/))&&d[0]&&(c[d[0].toLowerCase()]=c.ios)):((d=a.match(/NokiaN[^\/]*|Android \d\.\d|webOS\/\d\.\d/))&&(c.mobile=d[0]),/webOS/.test(a)&&(c.mobile="WebOS",(d=a.match(/webOS\/([^\s]*);/))&&d[1]&&(c.webos=b(d[1]))),/ Android/.test(a)&&(c.mobile="Android",(d=a.match(/Android ([^\s]*);/))&&d[1]&&(c.android=b(d[1])))),(d=a.match(/Chrome\/([^\s]*)/))&&d[1]?c.chrome=b(d[1]):(d=a.match(/AdobeAIR\/([^\s]*)/))&&(c.air=d[0])),c.webkit||((d=a.match(/Opera[\s\/]([^\s]*)/))&&d[1]?(c.opera=b(d[1]),(d=a.match(/Version\/([^\s]*)/))&&d[1]&&(c.opera=b(d[1])),(d=a.match(/Opera Mini[^;]*/))&&(c.mobile=d[0])):(d=a.match(/MSIE\s([^;]*)/))&&d[1]?c.ie=b(d[1]):(d=a.match(/Gecko\/([^\s]*)/))&&(c.gecko=1,(d=a.match(/rv:([^\s\)]*)/))&&d[1]&&(c.gecko=b(d[1]))))),c},L.env.ua=L.env.parseUA(),L.isFunction=function(a){return"function"==typeof a||"[object Function]"===N.toString.apply(a)},L._IEEnumFix=L.env.ua.ie?function(a,b){var c,d,e;for(c=0;c<O.length;c+=1)d=O[c],e=b[d],M.isFunction(e)&&e!=N[d]&&(a[d]=e)}:function(){},L.extend=function(a,b,c){if(!b||!a)throw Error("extend failed, please check that all dependencies are included.");var d,e=function(){};if(e.prototype=b.prototype,a.prototype=new e,a.prototype.constructor=a,a.superclass=b.prototype,b.prototype.constructor==N.constructor&&(b.prototype.constructor=b),c){for(d in c)M.hasOwnProperty(c,d)&&(a.prototype[d]=c[d]);M._IEEnumFix(a.prototype,c)}},"undefined"!=typeof KJUR&&KJUR||(KJUR={}),"undefined"!=typeof KJUR.asn1&&KJUR.asn1||(KJUR.asn1={}),KJUR.asn1.ASN1Util=new function(){this.integerToByteHex=function(a){return a=a.toString(16),1==a.length%2&&(a="0"+a),a},this.bigIntToMinTwosComplementsHex=function(a){var c=a.toString(16);if("-"!=c.substr(0,1))1==c.length%2?c="0"+c:c.match(/^[0-7]/)||(c="00"+c);else{var d=c.substr(1).length;1==d%2?d+=1:c.match(/^[0-7]/)||(d+=2);for(var c="",e=0;d>e;e++)c+="f";c=new b(c,16).xor(a).add(b.ONE).toString(16).replace(/^-/,"")}return c},this.getPEMStringFromHex=function(a,b){var c=CryptoJS.enc.Hex.parse(a),c=CryptoJS.enc.Base64.stringify(c).replace(/(.{64})/g,"$1\r\n"),c=c.replace(/\r\n$/,"");return"-----BEGIN "+b+"-----\r\n"+c+"\r\n-----END "+b+"-----\r\n"}},KJUR.asn1.ASN1Object=function(){this.getLengthHexFromValue=function(){if("undefined"==typeof this.hV||null==this.hV)throw"this.hV is null or undefined.";if(1==this.hV.length%2)throw"value hex must be even length: n=0,v="+this.hV;var a=this.hV.length/2,b=a.toString(16);if(1==b.length%2&&(b="0"+b),128>a)return b;var c=b.length/2;if(c>15)throw"ASN.1 length too long to represent by 8x: n = "+a.toString(16);return(128+c).toString(16)+b},this.getEncodedHex=function(){return(null==this.hTLV||this.isModified)&&(this.hV=this.getFreshValueHex(),this.hL=this.getLengthHexFromValue(),this.hTLV=this.hT+this.hL+this.hV,this.isModified=!1),this.hTLV},this.getValueHex=function(){return this.getEncodedHex(),this.hV},this.getFreshValueHex=function(){return""}},KJUR.asn1.DERAbstractString=function(a){KJUR.asn1.DERAbstractString.superclass.constructor.call(this),this.getString=function(){return this.s},this.setString=function(a){this.hTLV=null,this.isModified=!0,this.s=a,this.hV=stohex(this.s)},this.setStringHex=function(a){this.hTLV=null,this.isModified=!0,this.s=null,this.hV=a},this.getFreshValueHex=function(){return this.hV},"undefined"!=typeof a&&("undefined"!=typeof a.str?this.setString(a.str):"undefined"!=typeof a.hex&&this.setStringHex(a.hex))},L.extend(KJUR.asn1.DERAbstractString,KJUR.asn1.ASN1Object),KJUR.asn1.DERAbstractTime=function(){KJUR.asn1.DERAbstractTime.superclass.constructor.call(this),this.localDateToUTC=function(a){return utc=a.getTime()+6e4*a.getTimezoneOffset(),new Date(utc)},this.formatDate=function(a,b){var c=this.zeroPadding,d=this.localDateToUTC(a),e=String(d.getFullYear());"utc"==b&&(e=e.substr(2,2));var f=c(String(d.getMonth()+1),2),g=c(String(d.getDate()),2),h=c(String(d.getHours()),2),i=c(String(d.getMinutes()),2),c=c(String(d.getSeconds()),2);return e+f+g+h+i+c+"Z"},this.zeroPadding=function(a,b){return a.length>=b?a:Array(b-a.length+1).join("0")+a},this.getString=function(){return this.s},this.setString=function(a){this.hTLV=null,this.isModified=!0,this.s=a,this.hV=stohex(this.s)},this.setByDateValue=function(a,b,c,d,e,f){a=new Date(Date.UTC(a,b-1,c,d,e,f,0)),this.setByDate(a)},this.getFreshValueHex=function(){return this.hV}},L.extend(KJUR.asn1.DERAbstractTime,KJUR.asn1.ASN1Object),KJUR.asn1.DERAbstractStructured=function(a){KJUR.asn1.DERAbstractString.superclass.constructor.call(this),this.setByASN1ObjectArray=function(a){this.hTLV=null,this.isModified=!0,this.asn1Array=a},this.appendASN1Object=function(a){this.hTLV=null,this.isModified=!0,this.asn1Array.push(a)},this.asn1Array=[],"undefined"!=typeof a&&"undefined"!=typeof a.array&&(this.asn1Array=a.array)},L.extend(KJUR.asn1.DERAbstractStructured,KJUR.asn1.ASN1Object),KJUR.asn1.DERBoolean=function(){KJUR.asn1.DERBoolean.superclass.constructor.call(this),this.hT="01",this.hTLV="0101ff"},L.extend(KJUR.asn1.DERBoolean,KJUR.asn1.ASN1Object),KJUR.asn1.DERInteger=function(a){KJUR.asn1.DERInteger.superclass.constructor.call(this),this.hT="02",this.setByBigInteger=function(a){this.hTLV=null,this.isModified=!0,this.hV=KJUR.asn1.ASN1Util.bigIntToMinTwosComplementsHex(a)},this.setByInteger=function(a){a=new b(String(a),10),this.setByBigInteger(a)},this.setValueHex=function(a){this.hV=a},this.getFreshValueHex=function(){return this.hV},"undefined"!=typeof a&&("undefined"!=typeof a.bigint?this.setByBigInteger(a.bigint):"undefined"!=typeof a["int"]?this.setByInteger(a["int"]):"undefined"!=typeof a.hex&&this.setValueHex(a.hex))},L.extend(KJUR.asn1.DERInteger,KJUR.asn1.ASN1Object),KJUR.asn1.DERBitString=function(a){KJUR.asn1.DERBitString.superclass.constructor.call(this),this.hT="03",this.setHexValueIncludingUnusedBits=function(a){this.hTLV=null,this.isModified=!0,this.hV=a},this.setUnusedBitsAndHexValue=function(a,b){if(0>a||a>7)throw"unused bits shall be from 0 to 7: u = "+a;this.hTLV=null,this.isModified=!0,this.hV="0"+a+b},this.setByBinaryString=function(a){a=a.replace(/0+$/,"");var b=8-a.length%8;8==b&&(b=0);for(var c=0;b>=c;c++)a+="0";for(var d="",c=0;c<a.length-1;c+=8){var e=a.substr(c,8),e=parseInt(e,2).toString(16);1==e.length&&(e="0"+e),d+=e}this.hTLV=null,this.isModified=!0,this.hV="0"+b+d},this.setByBooleanArray=function(a){for(var b="",c=0;c<a.length;c++)b=1==a[c]?b+"1":b+"0";this.setByBinaryString(b)},this.newFalseArray=function(a){for(var b=Array(a),c=0;a>c;c++)b[c]=!1;return b},this.getFreshValueHex=function(){return this.hV},"undefined"!=typeof a&&("undefined"!=typeof a.hex?this.setHexValueIncludingUnusedBits(a.hex):"undefined"!=typeof a.bin?this.setByBinaryString(a.bin):"undefined"!=typeof a.array&&this.setByBooleanArray(a.array))},L.extend(KJUR.asn1.DERBitString,KJUR.asn1.ASN1Object),KJUR.asn1.DEROctetString=function(a){KJUR.asn1.DEROctetString.superclass.constructor.call(this,a),this.hT="04"},L.extend(KJUR.asn1.DEROctetString,KJUR.asn1.DERAbstractString),KJUR.asn1.DERNull=function(){KJUR.asn1.DERNull.superclass.constructor.call(this),this.hT="05",this.hTLV="0500"},L.extend(KJUR.asn1.DERNull,KJUR.asn1.ASN1Object),
KJUR.asn1.DERObjectIdentifier=function(a){var c=function(a){return a=a.toString(16),1==a.length&&(a="0"+a),a};KJUR.asn1.DERObjectIdentifier.superclass.constructor.call(this),this.hT="06",this.setValueHex=function(a){this.hTLV=null,this.isModified=!0,this.s=null,this.hV=a},this.setValueOidString=function(a){if(!a.match(/^[0-9.]+$/))throw"malformed oid string: "+a;var d="";a=a.split(".");var e=40*parseInt(a[0])+parseInt(a[1]),d=d+c(e);for(a.splice(0,2),e=0;e<a.length;e++){var f="",g=new b(a[e],10).toString(2),h=7-g.length%7;7==h&&(h=0);for(var i="",j=0;h>j;j++)i+="0";for(g=i+g,j=0;j<g.length-1;j+=7)h=g.substr(j,7),j!=g.length-7&&(h="1"+h),f+=c(parseInt(h,2));d+=f}this.hTLV=null,this.isModified=!0,this.s=null,this.hV=d},this.setValueName=function(a){if("undefined"==typeof KJUR.asn1.x509.OID.name2oidList[a])throw"DERObjectIdentifier oidName undefined: "+a;this.setValueOidString(KJUR.asn1.x509.OID.name2oidList[a])},this.getFreshValueHex=function(){return this.hV},"undefined"!=typeof a&&("undefined"!=typeof a.oid?this.setValueOidString(a.oid):"undefined"!=typeof a.hex?this.setValueHex(a.hex):"undefined"!=typeof a.name&&this.setValueName(a.name))},L.extend(KJUR.asn1.DERObjectIdentifier,KJUR.asn1.ASN1Object),KJUR.asn1.DERUTF8String=function(a){KJUR.asn1.DERUTF8String.superclass.constructor.call(this,a),this.hT="0c"},L.extend(KJUR.asn1.DERUTF8String,KJUR.asn1.DERAbstractString),KJUR.asn1.DERNumericString=function(a){KJUR.asn1.DERNumericString.superclass.constructor.call(this,a),this.hT="12"},L.extend(KJUR.asn1.DERNumericString,KJUR.asn1.DERAbstractString),KJUR.asn1.DERPrintableString=function(a){KJUR.asn1.DERPrintableString.superclass.constructor.call(this,a),this.hT="13"},L.extend(KJUR.asn1.DERPrintableString,KJUR.asn1.DERAbstractString),KJUR.asn1.DERTeletexString=function(a){KJUR.asn1.DERTeletexString.superclass.constructor.call(this,a),this.hT="14"},L.extend(KJUR.asn1.DERTeletexString,KJUR.asn1.DERAbstractString),KJUR.asn1.DERIA5String=function(a){KJUR.asn1.DERIA5String.superclass.constructor.call(this,a),this.hT="16"},L.extend(KJUR.asn1.DERIA5String,KJUR.asn1.DERAbstractString),KJUR.asn1.DERUTCTime=function(a){KJUR.asn1.DERUTCTime.superclass.constructor.call(this,a),this.hT="17",this.setByDate=function(a){this.hTLV=null,this.isModified=!0,this.date=a,this.s=this.formatDate(this.date,"utc"),this.hV=stohex(this.s)},"undefined"!=typeof a&&("undefined"!=typeof a.str?this.setString(a.str):"undefined"!=typeof a.hex?this.setStringHex(a.hex):"undefined"!=typeof a.date&&this.setByDate(a.date))},L.extend(KJUR.asn1.DERUTCTime,KJUR.asn1.DERAbstractTime),KJUR.asn1.DERGeneralizedTime=function(a){KJUR.asn1.DERGeneralizedTime.superclass.constructor.call(this,a),this.hT="18",this.setByDate=function(a){this.hTLV=null,this.isModified=!0,this.date=a,this.s=this.formatDate(this.date,"gen"),this.hV=stohex(this.s)},"undefined"!=typeof a&&("undefined"!=typeof a.str?this.setString(a.str):"undefined"!=typeof a.hex?this.setStringHex(a.hex):"undefined"!=typeof a.date&&this.setByDate(a.date))},L.extend(KJUR.asn1.DERGeneralizedTime,KJUR.asn1.DERAbstractTime),KJUR.asn1.DERSequence=function(a){KJUR.asn1.DERSequence.superclass.constructor.call(this,a),this.hT="30",this.getFreshValueHex=function(){for(var a="",b=0;b<this.asn1Array.length;b++)a+=this.asn1Array[b].getEncodedHex();return this.hV=a}},L.extend(KJUR.asn1.DERSequence,KJUR.asn1.DERAbstractStructured),KJUR.asn1.DERSet=function(a){KJUR.asn1.DERSet.superclass.constructor.call(this,a),this.hT="31",this.getFreshValueHex=function(){for(var a=[],b=0;b<this.asn1Array.length;b++)a.push(this.asn1Array[b].getEncodedHex());return a.sort(),this.hV=a.join("")}},L.extend(KJUR.asn1.DERSet,KJUR.asn1.DERAbstractStructured),KJUR.asn1.DERTaggedObject=function(a){KJUR.asn1.DERTaggedObject.superclass.constructor.call(this),this.hT="a0",this.hV="",this.isExplicit=!0,this.asn1Object=null,this.setASN1Object=function(a,b,c){this.hT=b,this.isExplicit=a,this.asn1Object=c,this.isExplicit?(this.hV=this.asn1Object.getEncodedHex(),this.hTLV=null,this.isModified=!0):(this.hV=null,this.hTLV=c.getEncodedHex(),this.hTLV=this.hTLV.replace(/^../,b),this.isModified=!1)},this.getFreshValueHex=function(){return this.hV},"undefined"!=typeof a&&("undefined"!=typeof a.tag&&(this.hT=a.tag),"undefined"!=typeof a.explicit&&(this.isExplicit=a.explicit),"undefined"!=typeof a.obj&&(this.asn1Object=a.obj,this.setASN1Object(this.isExplicit,this.hT,this.asn1Object)))},L.extend(KJUR.asn1.DERTaggedObject,KJUR.asn1.ASN1Object),function(a){var b,c={};c.decode=function(c){var d;if(b===a){var e="0123456789ABCDEF";for(b=[],d=0;16>d;++d)b[e.charAt(d)]=d;for(e=e.toLowerCase(),d=10;16>d;++d)b[e.charAt(d)]=d;for(d=0;8>d;++d)b[" \f\n\r	Â \u2028\u2029".charAt(d)]=-1}var e=[],f=0,g=0;for(d=0;d<c.length;++d){var h=c.charAt(d);if("="==h)break;if(h=b[h],-1!=h){if(h===a)throw"Illegal character at offset "+d;f|=h,2<=++g?(e[e.length]=f,g=f=0):f<<=4}}if(g)throw"Hex encoding incomplete: 4 bits missing";return e},window.Hex=c}(),function(a){var b,c={};c.decode=function(c){var d;if(b===a){for(b=[],d=0;64>d;++d)b["ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/".charAt(d)]=d;for(d=0;9>d;++d)b["= \f\n\r	Â \u2028\u2029".charAt(d)]=-1}var e=[],f=0,g=0;for(d=0;d<c.length;++d){var h=c.charAt(d);if("="==h)break;if(h=b[h],-1!=h){if(h===a)throw"Illegal character at offset "+d;f|=h,4<=++g?(e[e.length]=f>>16,e[e.length]=f>>8&255,e[e.length]=255&f,g=f=0):f<<=6}}switch(g){case 1:throw"Base64 encoding incomplete: at least 2 bits missing";case 2:e[e.length]=f>>10;break;case 3:e[e.length]=f>>16,e[e.length]=f>>8&255}return e},c.re=/-----BEGIN [^-]+-----([A-Za-z0-9+\/=\s]+)-----END [^-]+-----|begin-base64[^\n]+\n([A-Za-z0-9+\/=\s]+)====/,c.unarmor=function(a){var b=c.re.exec(a);if(b)if(b[1])a=b[1];else{if(!b[2])throw"RegExp out of sync";a=b[2]}return c.decode(a)},window.Base64=c}(),function(a){function b(a,c){a instanceof b?(this.enc=a.enc,this.pos=a.pos):(this.enc=a,this.pos=c)}function c(a,b,c,d,e){this.stream=a,this.header=b,this.length=c,this.tag=d,this.sub=e}var d={tag:function(a,b){var c=document.createElement(a);return c.className=b,c},text:function(a){return document.createTextNode(a)}};b.prototype.get=function(b){if(b===a&&(b=this.pos++),b>=this.enc.length)throw"Requesting byte offset "+b+" on a stream of length "+this.enc.length;return this.enc[b]},b.prototype.hexDigits="0123456789ABCDEF",b.prototype.hexByte=function(a){return this.hexDigits.charAt(a>>4&15)+this.hexDigits.charAt(15&a)},b.prototype.hexDump=function(a,b,c){for(var d="";b>a;++a)if(d+=this.hexByte(this.get(a)),!0!==c)switch(15&a){case 7:d+="  ";break;case 15:d+="\n";break;default:d+=" "}return d},b.prototype.parseStringISO=function(a,b){for(var c="",d=a;b>d;++d)c+=String.fromCharCode(this.get(d));return c},b.prototype.parseStringUTF=function(a,b){for(var c="",d=a;b>d;)var e=this.get(d++),c=128>e?c+String.fromCharCode(e):e>191&&224>e?c+String.fromCharCode((31&e)<<6|63&this.get(d++)):c+String.fromCharCode((15&e)<<12|(63&this.get(d++))<<6|63&this.get(d++));return c},b.prototype.parseStringBMP=function(a,b){for(var c="",d=a;b>d;d+=2)var e=this.get(d),f=this.get(d+1),c=c+String.fromCharCode((e<<8)+f);return c},b.prototype.reTime=/^((?:1[89]|2\d)?\d\d)(0[1-9]|1[0-2])(0[1-9]|[12]\d|3[01])([01]\d|2[0-3])(?:([0-5]\d)(?:([0-5]\d)(?:[.,](\d{1,3}))?)?)?(Z|[-+](?:[0]\d|1[0-2])([0-5]\d)?)?$/,b.prototype.parseTime=function(a,b){var c=this.parseStringISO(a,b),d=this.reTime.exec(c);return d?(c=d[1]+"-"+d[2]+"-"+d[3]+" "+d[4],d[5]&&(c+=":"+d[5],d[6]&&(c+=":"+d[6],d[7]&&(c+="."+d[7]))),d[8]&&(c+=" UTC","Z"!=d[8]&&(c+=d[8],d[9]&&(c+=":"+d[9]))),c):"Unrecognized time: "+c},b.prototype.parseInteger=function(a,b){var c=b-a;if(c>4){var c=c<<3,d=this.get(a);if(0===d)c-=8;else for(;128>d;)d<<=1,--c;return"("+c+" bit)"}for(c=0,d=a;b>d;++d)c=c<<8|this.get(d);return c},b.prototype.parseBitString=function(a,b){var c=this.get(a),d=(b-a-1<<3)-c,e="("+d+" bit)";if(20>=d)for(var f=c,e=e+" ",c=b-1;c>a;--c){for(d=this.get(c);8>f;++f)e+=d>>f&1?"1":"0";f=0}return e},b.prototype.parseOctetString=function(a,b){var c=b-a,d="("+c+" byte) ";c>100&&(b=a+100);for(var e=a;b>e;++e)d+=this.hexByte(this.get(e));return c>100&&(d+="â€¦"),d},b.prototype.parseOID=function(a,b){for(var c="",d=0,e=0,f=a;b>f;++f){var g=this.get(f),d=d<<7|127&g,e=e+7;128&g||(""===c?(c=80>d?40>d?0:1:2,c=c+"."+(d-40*c)):c+="."+(e>=31?"bigint":d),d=e=0)}return c},c.prototype.typeName=function(){if(this.tag===a)return"unknown";var b=31&this.tag;switch(this.tag>>6){case 0:switch(b){case 0:return"EOC";case 1:return"BOOLEAN";case 2:return"INTEGER";case 3:return"BIT_STRING";case 4:return"OCTET_STRING";case 5:return"NULL";case 6:return"OBJECT_IDENTIFIER";case 7:return"ObjectDescriptor";case 8:return"EXTERNAL";case 9:return"REAL";case 10:return"ENUMERATED";case 11:return"EMBEDDED_PDV";case 12:return"UTF8String";case 16:return"SEQUENCE";case 17:return"SET";case 18:return"NumericString";case 19:return"PrintableString";case 20:return"TeletexString";case 21:return"VideotexString";case 22:return"IA5String";case 23:return"UTCTime";case 24:return"GeneralizedTime";case 25:return"GraphicString";case 26:return"VisibleString";case 27:return"GeneralString";case 28:return"UniversalString";case 30:return"BMPString";default:return"Universal_"+b.toString(16)}case 1:return"Application_"+b.toString(16);case 2:return"["+b+"]";case 3:return"Private_"+b.toString(16)}},c.prototype.reSeemsASCII=/^[ -~]+$/,c.prototype.content=function(){if(this.tag===a)return null;var b=this.tag>>6,c=31&this.tag,d=this.posContent(),e=Math.abs(this.length);if(0!==b)return null!==this.sub?"("+this.sub.length+" elem)":(b=this.stream.parseStringISO(d,d+Math.min(e,100)),this.reSeemsASCII.test(b)?b.substring(0,200)+(200<b.length?"â€¦":""):this.stream.parseOctetString(d,d+e));switch(c){case 1:return 0===this.stream.get(d)?"false":"true";case 2:return this.stream.parseInteger(d,d+e);case 3:return this.sub?"("+this.sub.length+" elem)":this.stream.parseBitString(d,d+e);case 4:return this.sub?"("+this.sub.length+" elem)":this.stream.parseOctetString(d,d+e);case 6:return this.stream.parseOID(d,d+e);case 16:case 17:return"("+this.sub.length+" elem)";case 12:return this.stream.parseStringUTF(d,d+e);case 18:case 19:case 20:case 21:case 22:case 26:return this.stream.parseStringISO(d,d+e);case 30:return this.stream.parseStringBMP(d,d+e);case 23:case 24:return this.stream.parseTime(d,d+e)}return null},c.prototype.toString=function(){return this.typeName()+"@"+this.stream.pos+"[header:"+this.header+",length:"+this.length+",sub:"+(null===this.sub?"null":this.sub.length)+"]"},c.prototype.print=function(b){if(b===a&&(b=""),document.writeln(b+this),null!==this.sub){b+="  ";for(var c=0,d=this.sub.length;d>c;++c)this.sub[c].print(b)}},c.prototype.toPrettyString=function(b){b===a&&(b="");var c=b+this.typeName()+" @"+this.stream.pos;if(0<=this.length&&(c+="+"),c+=this.length,32&this.tag?c+=" (constructed)":3!=this.tag&&4!=this.tag||null===this.sub||(c+=" (encapsulates)"),c+="\n",null!==this.sub){b+="  ";for(var d=0,e=this.sub.length;e>d;++d)c+=this.sub[d].toPrettyString(b)}return c},c.prototype.toDOM=function(){var a=d.tag("div","node");a.asn1=this;var b=d.tag("div","head"),c=this.typeName().replace(/_/g," ");b.innerHTML=c;var e=this.content();null!==e&&(e=String(e).replace(/</g,"&lt;"),c=d.tag("span","preview"),c.appendChild(d.text(e)),b.appendChild(c)),a.appendChild(b),this.node=a,this.head=b;var f=d.tag("div","value"),c="Offset: "+this.stream.pos+"<br/>",c=c+("Length: "+this.header+"+"),c=0<=this.length?c+this.length:c+(-this.length+" (undefined)");if(32&this.tag?c+="<br/>(constructed)":3!=this.tag&&4!=this.tag||null===this.sub||(c+="<br/>(encapsulates)"),null!==e&&(c+="<br/>Value:<br/><b>"+e+"</b>","object"==typeof oids&&6==this.tag&&(e=oids[e]))&&(e.d&&(c+="<br/>"+e.d),e.c&&(c+="<br/>"+e.c),e.w&&(c+="<br/>(warning!)")),f.innerHTML=c,a.appendChild(f),c=d.tag("div","sub"),null!==this.sub)for(e=0,f=this.sub.length;f>e;++e)c.appendChild(this.sub[e].toDOM());return a.appendChild(c),b.onclick=function(){a.className="node collapsed"==a.className?"node":"node collapsed"},a},c.prototype.posStart=function(){return this.stream.pos},c.prototype.posContent=function(){return this.stream.pos+this.header},c.prototype.posEnd=function(){return this.stream.pos+this.header+Math.abs(this.length)},c.prototype.fakeHover=function(a){this.node.className+=" hover",a&&(this.head.className+=" hover")},c.prototype.fakeOut=function(a){var b=/ ?hover/;this.node.className=this.node.className.replace(b,""),a&&(this.head.className=this.head.className.replace(b,""))},c.prototype.toHexDOM_sub=function(a,b,c,e,f){e>=f||(b=d.tag("span",b),b.appendChild(d.text(c.hexDump(e,f))),a.appendChild(b))},c.prototype.toHexDOM=function(b){var c=d.tag("span","hex");if(b===a&&(b=c),this.head.hexNode=c,this.head.onmouseover=function(){this.hexNode.className="hexCurrent"},this.head.onmouseout=function(){this.hexNode.className="hex"},c.asn1=this,c.onmouseover=function(){var a=!b.selected;a&&(b.selected=this.asn1,this.className="hexCurrent"),this.asn1.fakeHover(a)},c.onmouseout=function(){var a=b.selected==this.asn1;this.asn1.fakeOut(a),a&&(b.selected=null,this.className="hex")},this.toHexDOM_sub(c,"tag",this.stream,this.posStart(),this.posStart()+1),this.toHexDOM_sub(c,0<=this.length?"dlen":"ulen",this.stream,this.posStart()+1,this.posContent()),null===this.sub)c.appendChild(d.text(this.stream.hexDump(this.posContent(),this.posEnd())));else if(0<this.sub.length){var e=this.sub[0],f=this.sub[this.sub.length-1];this.toHexDOM_sub(c,"intro",this.stream,this.posContent(),e.posStart());for(var e=0,g=this.sub.length;g>e;++e)c.appendChild(this.sub[e].toHexDOM(b));this.toHexDOM_sub(c,"outro",this.stream,f.posEnd(),this.posEnd())}return c},c.prototype.toHexString=function(){return this.stream.hexDump(this.posStart(),this.posEnd(),!0)},c.decodeLength=function(a){var b=a.get(),c=127&b;if(c==b)return c;if(c>3)throw"Length over 24 bits not supported at position "+(a.pos-1);if(0===c)return-1;for(var d=b=0;c>d;++d)b=b<<8|a.get();return b},c.hasContent=function(a,d,e){if(32&a)return!0;if(3>a||a>4)return!1;var f=new b(e);if(3==a&&f.get(),f.get()>>6&1)return!1;try{var g=c.decodeLength(f);return f.pos-e.pos+g==d}catch(h){return!1}},c.decode=function(a){a instanceof b||(a=new b(a,0));var d=new b(a),e=a.get(),f=c.decodeLength(a),g=a.pos-d.pos,h=null;if(c.hasContent(e,f,a)){var i=a.pos;if(3==e&&a.get(),h=[],f>=0){for(var j=i+f;a.pos<j;)h[h.length]=c.decode(a);if(a.pos!=j)throw"Content size is not correct for container starting at offset "+i}else try{for(;j=c.decode(a),0!==j.tag;)h[h.length]=j;f=i-a.pos}catch(k){throw"Exception while decoding undefined length content: "+k}}else a.pos+=f;return new c(d,g,f,e,h)},c.test=function(){for(var a=[{value:[39],expected:39},{value:[129,201],expected:201},{value:[131,254,220,186],expected:16702650}],d=0,e=a.length;e>d;++d){var f=new b(a[d].value,0),f=c.decodeLength(f);f!=a[d].expected&&document.write("In test["+d+"] expected "+a[d].expected+" got "+f+"\n")}},window.ASN1=c}(),ASN1.prototype.getHexStringValue=function(){return this.toHexString().substr(2*this.header,2*this.length)},w.prototype.parseKey=function(a){try{var b=/^\s*(?:[0-9A-Fa-f][0-9A-Fa-f]\s*)+$/.test(a)?Hex.decode(a):Base64.unarmor(a),c=ASN1.decode(b);if(9===c.sub.length){var d=c.sub[1].getHexStringValue();this.n=u(d,16);var e=c.sub[2].getHexStringValue();this.e=parseInt(e,16);var f=c.sub[3].getHexStringValue();this.d=u(f,16);var g=c.sub[4].getHexStringValue();this.p=u(g,16);var h=c.sub[5].getHexStringValue();this.q=u(h,16);var i=c.sub[6].getHexStringValue();this.dmp1=u(i,16);var j=c.sub[7].getHexStringValue();this.dmq1=u(j,16);var k=c.sub[8].getHexStringValue();this.coeff=u(k,16)}else{if(2!==c.sub.length)return!1;var l=c.sub[1].sub[0],d=l.sub[0].getHexStringValue();this.n=u(d,16),e=l.sub[1].getHexStringValue(),this.e=parseInt(e,16)}return!0}catch(m){return!1}},w.prototype.getPrivateBaseKey=function(){var a={array:[new KJUR.asn1.DERInteger({"int":0}),new KJUR.asn1.DERInteger({bigint:this.n}),new KJUR.asn1.DERInteger({"int":this.e}),new KJUR.asn1.DERInteger({bigint:this.d}),new KJUR.asn1.DERInteger({bigint:this.p}),new KJUR.asn1.DERInteger({bigint:this.q}),new KJUR.asn1.DERInteger({bigint:this.dmp1}),new KJUR.asn1.DERInteger({bigint:this.dmq1}),new KJUR.asn1.DERInteger({bigint:this.coeff})]};return new KJUR.asn1.DERSequence(a).getEncodedHex()},w.prototype.getPrivateBaseKeyB64=function(){return x(this.getPrivateBaseKey())},w.prototype.getPublicBaseKey=function(){var a={array:[new KJUR.asn1.DERObjectIdentifier({oid:"1.2.840.113549.1.1.1"}),new KJUR.asn1.DERNull]},b=new KJUR.asn1.DERSequence(a),a={array:[new KJUR.asn1.DERInteger({bigint:this.n}),new KJUR.asn1.DERInteger({"int":this.e})]},a={hex:"00"+new KJUR.asn1.DERSequence(a).getEncodedHex()},a=new KJUR.asn1.DERBitString(a),a={array:[b,a]};return new KJUR.asn1.DERSequence(a).getEncodedHex()},w.prototype.getPublicBaseKeyB64=function(){return x(this.getPublicBaseKey())},w.prototype.wordwrap=function(a,b){return b=b||64,a?a.match(RegExp("(.{1,"+b+"})( +|$\n?)|(.{1,"+b+"})","g")).join("\n"):a},w.prototype.getPrivateKey=function(){var a;return a="-----BEGIN RSA PRIVATE KEY-----\n"+(this.wordwrap(this.getPrivateBaseKeyB64())+"\n"),a+"-----END RSA PRIVATE KEY-----"},w.prototype.getPublicKey=function(){var a;return a="-----BEGIN PUBLIC KEY-----\n"+(this.wordwrap(this.getPublicBaseKeyB64())+"\n"),a+"-----END PUBLIC KEY-----"},w.prototype.hasPublicKeyProperty=function(a){return a=a||{},a.hasOwnProperty("n")&&a.hasOwnProperty("e")},w.prototype.hasPrivateKeyProperty=function(a){return a=a||{},a.hasOwnProperty("n")&&a.hasOwnProperty("e")&&a.hasOwnProperty("d")&&a.hasOwnProperty("p")&&a.hasOwnProperty("q")&&a.hasOwnProperty("dmp1")&&a.hasOwnProperty("dmq1")&&a.hasOwnProperty("coeff")},w.prototype.parsePropertiesFrom=function(a){this.n=a.n,this.e=a.e,a.hasOwnProperty("d")&&(this.d=a.d,this.p=a.p,this.q=a.q,this.dmp1=a.dmp1,this.dmq1=a.dmq1,this.coeff=a.coeff)};var P=function(a){w.call(this),a&&("string"==typeof a?this.parseKey(a):(this.hasPrivateKeyProperty(a)||this.hasPublicKeyProperty(a))&&this.parsePropertiesFrom(a))};P.prototype=new w,P.prototype.constructor=P,L=function(a){a=a||{},this.default_key_size=parseInt(a.default_key_size)||1024,this.default_public_exponent=a.default_public_exponent||"010001",this.log=a.log||!1,this.key=null},L.prototype.setKey=function(a){this.log&&this.key&&console.warn("A key was already set, overriding existing."),this.key=new P(a)},L.prototype.setPrivateKey=function(a){this.setKey(a)},L.prototype.setPublicKey=function(a){this.setKey(a)},L.prototype.decrypt=function(a){try{return this.getKey().decrypt(y(a))}catch(b){return!1}},L.prototype.encrypt=function(a){try{return x(this.getKey().encrypt(a))}catch(b){return!1}},L.prototype.getKey=function(a){if(!this.key){if(this.key=new P,a&&"[object Function]"==={}.toString.call(a))return void this.key.generateAsync(this.default_key_size,this.default_public_exponent,a);this.key.generate(this.default_key_size,this.default_public_exponent)}return this.key},L.prototype.getPrivateKey=function(){return this.getKey().getPrivateKey()},L.prototype.getPrivateKeyB64=function(){return this.getKey().getPrivateBaseKeyB64()},L.prototype.getPublicKey=function(){return this.getKey().getPublicKey()},L.prototype.getPublicKeyB64=function(){return this.getKey().getPublicBaseKeyB64()},a.JSEncrypt=L}(JSEncryptExports);var JSEncrypt=JSEncryptExports.JSEncrypt;!function(a){function b(){return this instanceof b?void 0:new b}var c=a.Moip||{};a.Moip=c,b.prototype={calculateAccount:function(a){for(var b=a.split(""),c=0,d=0;d<b.length;d++)seq=9-d,c+=parseInt(b[d])*seq;return this.module(c)},calculateAgency:function(a){for(var b=a.split(""),c=0,d=0;d<b.length;d++)seq=5-d,c+=parseInt(b[d])*seq;return this.module(c)},module:function(a){var b=11-a%11;return 10===b?"X":11===b?"0":b.toString()}},c.BancoDoBrasilCheckNumberCalculator=b()}(window),function(a){function b(){return this instanceof b?void 0:new b}var c=a.Moip||{};a.Moip=c,b.prototype={agencyNumberIsValid:function(a){return c.CommonBankAccountValidator.agencyNumberIsValid(a)},agencyCheckNumberIsValid:function(a){return a.length==this.agencyCheckNumberLength()&&c.CommonBankAccountValidator.agencyCheckNumberIsValid(a)},accountNumberIsValid:function(a){return a.length==this.accountNumberLength()&&c.CommonBankAccountValidator.accountNumberIsValid(a)},accountCheckNumberIsValid:function(a){return c.CommonBankAccountValidator.accountCheckNumberIsValid(a)},agencyCheckNumberMatch:function(a){var b=c.BancoDoBrasilCheckNumberCalculator.calculateAgency(a.agencyNumber);return b===a.agencyCheckNumber.toUpperCase()},accountCheckNumberMatch:function(a){var b=c.BancoDoBrasilCheckNumberCalculator.calculateAccount(a.accountNumber);return b===a.accountCheckNumber.toUpperCase()},agencyNumberMsgError:function(){return c.CommonBankAccountValidator.agencyNumberMsgError()},agencyCheckNumberMsgError:function(){return c.CommonBankAccountValidator.agencyCheckNumberMsgError(this.agencyCheckNumberLength())},accountNumberMsgError:function(){return c.CommonBankAccountValidator.accountNumberMsgError(this.accountNumberLength())},agencyCheckNumberLength:function(){return 1},accountNumberLength:function(){return 8}},c.BancoDoBrasilValidator=b()}(window),function(a){function b(){return this instanceof b?void 0:new b}var c=a.Moip||{};a.Moip=c,b.prototype={validator:function(a){var b={"001":c.BancoDoBrasilValidator,237:c.BradescoValidator,341:c.ItauValidator,"033":c.SantanderValidator,"041":c.BanrisulValidator,745:c.CitibankValidator,399:c.HSBCValidator};return b[a]?b[a]:c.GenericBankAccountValidator},validate:function(a){var b=[],d=this.validator(a.bankNumber);c.GenericBankAccountValidator.bankNumberIsValid(a.bankNumber)||b.push({description:"Banco invÃ¡lido",code:"INVALID_BANK_NUMBER"}),d.agencyNumberIsValid(a.agencyNumber)||b.push({description:d.agencyNumberMsgError(),code:"INVALID_AGENCY_NUMBER"}),d.agencyCheckNumberIsValid(a.agencyCheckNumber)||b.push({description:d.agencyCheckNumberMsgError(),code:"INVALID_AGENCY_CHECK_NUMBER"}),d.accountNumberIsValid(a.accountNumber)||b.push({description:d.accountNumberMsgError(),code:"INVALID_ACCOUNT_NUMBER"}),d.accountCheckNumberIsValid(a.accountCheckNumber)||b.push({description:"DÃ­gito da conta corrente invÃ¡lido",code:"INVALID_ACCOUNT_CHECK_NUMBER"}),d.agencyNumberIsValid(a.agencyNumber)&&d.agencyCheckNumberIsValid(a.agencyCheckNumber)&&(d.agencyCheckNumberMatch(a)||b.push({description:"DÃ­gito da agÃªncia nÃ£o corresponde ao nÃºmero da agÃªncia preenchido",code:"AGENCY_CHECK_NUMBER_DONT_MATCH"})),d.accountNumberIsValid(a.accountNumber)&&d.accountCheckNumberIsValid(a.accountCheckNumber)&&(d.accountCheckNumberMatch(a)||b.push({description:"DÃ­gito da conta nÃ£o corresponde ao nÃºmero da conta/agÃªncia preenchido",code:"ACCOUNT_CHECK_NUMBER_DONT_MATCH"})),0===b.length?a.valid():a.invalid({errors:b})}},c.BankAccount=b()}(window),function(a){function b(){return this instanceof b?void 0:new b}var c=a.Moip||{};a.Moip=c,b.prototype={calculateAccount:function(a){for(var b=a.split(""),c=0,d=0;d<b.length;d++){var e=parseInt(b[d]);c+=this.multiplyAccordingWeight(e,d)}return this.moduleAccount(c)},multiplyAccordingWeight:function(a,b){var c=[3,2,4,7,6,5,4,3,2];return a*c[b]},moduleAccount:function(a){var b=a%11;return 0===b?"0":1===b?"6":(11-b).toString()},calculateAgency:function(a){var b=a.split(""),c=this.firstAgencyDigit(b),d=this.secondAgencyDigit(b,c);return c+d},firstAgencyDigit:function(a){for(var b=0,c=0,d=0;d<a.length;d++){var e=parseInt(a[d]);c=this.multiplyAccordingParity(e,d),c=this.adjustAccordingLength(c),b+=c}return this.moduleAgencyFirstDigit(b)},secondAgencyDigit:function(a,b){a.push(b);for(var c=0,d=0;d<a.length;d++)seq=6-d,c+=parseInt(a[d])*seq;var e=this.moduleAgencySecondDigit(c);return"1"===e&&(9!=b?b+=1:b=0,this.secondAgencyDigit(a,b)),e},multiplyAccordingParity:function(a,b){return a*(b%2===0?1:2)},adjustAccordingLength:function(a){if(a>9){var b=a.toString().split("");a=0;for(var c=0;c<b.length;c++)a+=parseInt(b[c])}return a},moduleAgencyFirstDigit:function(a){var b=a%10;return 0===b?"0":(10-b).toString()},moduleAgencySecondDigit:function(a){var b=a%11;return 0===b?"0":1===b?"1":(11-b).toString()}},c.BanrisulCheckNumberCalculator=b()}(window),function(a){function b(){return this instanceof b?void 0:new b}var c=a.Moip||{};a.Moip=c,b.prototype={agencyNumberIsValid:function(a){return c.CommonBankAccountValidator.agencyNumberIsValid(a)},agencyCheckNumberIsValid:function(a){return a.length==this.agencyCheckNumberLength()&&c.GenericBankAccountValidator.agencyCheckNumberIsValid(a)},accountNumberIsValid:function(a){return a.length==this.accountNumberLength()&&c.CommonBankAccountValidator.accountNumberIsValid(a)},accountCheckNumberIsValid:function(a){return c.CommonBankAccountValidator.accountCheckNumberIsValid(a)},agencyCheckNumberMatch:function(a){var b=c.BanrisulCheckNumberCalculator.calculateAgency(a.agencyNumber);return b===a.agencyCheckNumber},accountCheckNumberMatch:function(a){var b=c.BanrisulCheckNumberCalculator.calculateAccount(a.accountNumber);return b===a.accountCheckNumber},agencyNumberMsgError:function(){return c.CommonBankAccountValidator.agencyNumberMsgError()},agencyCheckNumberMsgError:function(){return c.CommonBankAccountValidator.agencyCheckNumberMsgError(this.agencyCheckNumberLength())},accountNumberMsgError:function(){return c.CommonBankAccountValidator.accountNumberMsgError(this.accountNumberLength())},agencyCheckNumberLength:function(){return 2},accountNumberLength:function(){return 9}},c.BanrisulValidator=b()}(window),function(a){function b(){return this instanceof b?void 0:new b}var c=a.Moip||{};a.Moip=c,b.prototype={calculateAccount:function(a){for(var b=a.split(""),c=0,d=0;d<b.length;d++){var e=parseInt(b[d]);c+=this.multiplyAccordingWeight(e,d)}return this.accountModule(c)},multiplyAccordingWeight:function(a,b){var c=[2,7,6,5,4,3,2];return a*c[b]},accountModule:function(a){var b=a%11;return 0===b?"0":1===b?"P":(11-b).toString()},calculateAgency:function(a){for(var b=a.split(""),c=0,d=0;d<b.length;d++)seq=5-d,c+=parseInt(b[d])*seq;return this.agencyModule(c)},agencyModule:function(a){var b=11-a%11;return 10===b?"P":11===b?"0":b.toString()}},c.BradescoCheckNumberCalculator=b()}(window),function(a){function b(){return this instanceof b?void 0:new b}var c=a.Moip||{};a.Moip=c,b.prototype={agencyNumberIsValid:function(a){return c.CommonBankAccountValidator.agencyNumberIsValid(a)},agencyCheckNumberIsValid:function(a){return a.length==this.agencyCheckNumberLength()&&c.CommonBankAccountValidator.agencyCheckNumberIsValid(a)},accountNumberIsValid:function(a){return a.length==this.accountNumberLength()&&c.CommonBankAccountValidator.accountNumberIsValid(a)},accountCheckNumberIsValid:function(a){return c.CommonBankAccountValidator.accountCheckNumberIsValid(a)},agencyCheckNumberMatch:function(a){var b=c.BradescoCheckNumberCalculator.calculateAgency(a.agencyNumber),d=a.agencyCheckNumber.toUpperCase();return"0"===d?b===d||"P"===b:b===d},accountCheckNumberMatch:function(a){var b=c.BradescoCheckNumberCalculator.calculateAccount(a.accountNumber),d=a.accountCheckNumber.toUpperCase();return"0"===d?b===d||"P"===b:b===d},agencyNumberMsgError:function(){return c.CommonBankAccountValidator.agencyNumberMsgError()},agencyCheckNumberMsgError:function(){return c.CommonBankAccountValidator.agencyCheckNumberMsgError(this.agencyCheckNumberLength())},accountNumberMsgError:function(){return c.CommonBankAccountValidator.accountNumberMsgError(this.accountNumberLength())},agencyCheckNumberLength:function(){return 1},accountNumberLength:function(){return 7}},c.BradescoValidator=b()}(window),function(a){function b(){return this instanceof b?void 0:new b}var c=a.Moip||{};a.Moip=c,b.prototype={agencyNumberIsValid:function(a){return c.CommonBankAccountValidator.agencyNumberIsValid(a)},agencyCheckNumberIsValid:function(a){return void 0===a||""===a},accountNumberIsValid:function(a){return a.length==this.accountNumberLength()&&c.CommonBankAccountValidator.accountNumberIsValid(a)},accountCheckNumberIsValid:function(a){return c.CommonBankAccountValidator.accountCheckNumberIsValid(a)},agencyCheckNumberMatch:function(){return!0},accountCheckNumberMatch:function(){return!0},agencyNumberMsgError:function(){return c.CommonBankAccountValidator.agencyNumberMsgError()},agencyCheckNumberMsgError:function(){return c.CommonBankAccountValidator.agencyCheckNumberMsgError()},accountNumberMsgError:function(){return c.CommonBankAccountValidator.accountNumberMsgError(this.accountNumberLength())},accountNumberLength:function(){return 7}},c.CitibankValidator=b()}(window),function(a){function b(){return this instanceof b?void 0:new b}var c=a.Moip||{};a.Moip=c,b.prototype={agencyNumberIsValid:function(a){return/^(?!0000)([0-9]{4})$/.test(a)},agencyCheckNumberIsValid:function(a){return/^[a-zA-Z0-9]{0,1}$/.test(a)},accountNumberIsValid:function(a){return/^[0-9]{1,12}$/.test(a)&&parseInt(a)>0},accountCheckNumberIsValid:function(a){return/^[a-zA-Z0-9]{1}$/.test(a)},agencyNumberMsgError:function(a){return void 0===a&&(a=this.agencyNumberLength()),"A agÃªncia deve conter "+a+" nÃºmeros. Complete com zeros a esquerda se necessÃ¡rio."},agencyCheckNumberMsgError:function(a){return void 0===a||0===a?"O dÃ­gito da agÃªncia deve ser vazio":1===a?"O dÃ­gito da agÃªncia deve conter 1 dÃ­gito":"O dÃ­gito da agÃªncia deve conter "+a+" nÃºmeros. Complete com zeros a esquerda se necessÃ¡rio."},accountNumberMsgError:function(a){return"A conta corrente deve conter "+a+" nÃºmeros. Complete com zeros a esquerda se necessÃ¡rio."},agencyNumberLength:function(){return 4}},c.CommonBankAccountValidator=b()}(window),function(a){function b(){return this instanceof b?void 0:new b}var c=a.Moip||{};a.Moip=c,b.prototype={bankNumberIsValid:function(a){return/^([0-9-A-Za-x]{3,5})$/.test(a)},agencyNumberIsValid:function(a){return/^[0-9]{1,5}$/.test(a)&&parseInt(a)>0},agencyCheckNumberIsValid:function(a){return/^[a-zA-Z0-9]{0,2}$/.test(a)},accountNumberIsValid:function(a){return/^[0-9]{1,12}$/.test(a)&&parseInt(a)>0},accountCheckNumberIsValid:function(a){return/^[a-zA-Z0-9]{0,2}$/.test(a)},agencyCheckNumberMatch:function(){return!0},accountCheckNumberMatch:function(){return!0},agencyNumberMsgError:function(){return"AgÃªncia invÃ¡lida"},agencyCheckNumberMsgError:function(){return"DÃ­gito da agÃªncia invÃ¡lido"},accountNumberMsgError:function(){return"Conta corrente invÃ¡lida"}},c.GenericBankAccountValidator=b()}(window),function(a){function b(){return this instanceof b?void 0:new b}var c=a.Moip||{};a.Moip=c,b.prototype={agencyNumberIsValid:function(a){return c.CommonBankAccountValidator.agencyNumberIsValid(a)},agencyCheckNumberIsValid:function(a){return void 0===a||""===a},accountNumberIsValid:function(a){return a.length==this.accountNumberLength()&&c.CommonBankAccountValidator.accountNumberIsValid(a)},accountCheckNumberIsValid:function(){return!0},agencyCheckNumberMatch:function(){return!0},accountCheckNumberMatch:function(){return!0},agencyNumberMsgError:function(){return c.CommonBankAccountValidator.agencyNumberMsgError()},agencyCheckNumberMsgError:function(){return c.CommonBankAccountValidator.agencyCheckNumberMsgError()},accountNumberMsgError:function(){return c.CommonBankAccountValidator.accountNumberMsgError(this.accountNumberLength())},accountNumberLength:function(){return 6}},c.HSBCValidator=b()}(window),function(a){function b(){return this instanceof b?void 0:new b}var c=a.Moip||{};a.Moip=c,b.prototype={calculate:function(a,b){for(var c=(a+b).split(""),d=0,e=0,f=0;f<c.length;f++){var g=parseInt(c[f]);e=this.multiplyAccordingParity(g,f),e=this.adjustAccordingLength(e),d+=e}return this.module(d)},multiplyAccordingParity:function(a,b){return a*(b%2===0?2:1)},adjustAccordingLength:function(a){if(a>9){var b=a.toString().split("");a=0;for(var c=0;c<b.length;c++)a+=parseInt(b[c])}return a},module:function(a){var b=a%10;return 0===b?"0":(10-b).toString()}},c.ItauCheckNumberCalculator=b()}(window),function(a){function b(){return this instanceof b?void 0:new b;

}var c=a.Moip||{};a.Moip=c,b.prototype={agencyNumberIsValid:function(a){return c.CommonBankAccountValidator.agencyNumberIsValid(a)},agencyCheckNumberIsValid:function(a){return void 0===a||""===a},accountNumberIsValid:function(a){return a.length==this.accountNumberLength()&&c.CommonBankAccountValidator.accountNumberIsValid(a)},accountCheckNumberIsValid:function(a){return c.CommonBankAccountValidator.accountCheckNumberIsValid(a)},agencyCheckNumberMatch:function(){return!0},accountCheckNumberMatch:function(a){var b=c.ItauCheckNumberCalculator.calculate(a.agencyNumber,a.accountNumber);return b===a.accountCheckNumber},agencyNumberMsgError:function(){return c.CommonBankAccountValidator.agencyNumberMsgError()},agencyCheckNumberMsgError:function(){return c.CommonBankAccountValidator.agencyCheckNumberMsgError()},accountNumberMsgError:function(){return c.CommonBankAccountValidator.accountNumberMsgError(this.accountNumberLength())},accountNumberLength:function(){return 5}},c.ItauValidator=b()}(window),function(a){function b(){return this instanceof b?void 0:new b}var c=a.Moip||{};a.Moip=c,b.prototype={agencyNumberIsValid:function(a){return c.CommonBankAccountValidator.agencyNumberIsValid(a)},agencyCheckNumberIsValid:function(a){return void 0===a||""===a},accountNumberIsValid:function(a){return a.length==this.accountNumberLength()&&c.CommonBankAccountValidator.accountNumberIsValid(a)},accountCheckNumberIsValid:function(a){return c.CommonBankAccountValidator.accountCheckNumberIsValid(a)},agencyCheckNumberMatch:function(){return!0},accountCheckNumberMatch:function(){return!0},agencyNumberMsgError:function(){return c.CommonBankAccountValidator.agencyNumberMsgError()},agencyCheckNumberMsgError:function(){return c.CommonBankAccountValidator.agencyCheckNumberMsgError()},accountNumberMsgError:function(){return c.CommonBankAccountValidator.accountNumberMsgError(this.accountNumberLength())},accountNumberLength:function(){return 8}},c.SantanderValidator=b()}(window),function(a){var b=a.Moip||{};a.Moip=b,b.Calculator={pricing:function(a){return this.buildJson(a)},buildJson:function(a){var b,c=[],d=[],e=[],f=[];if(b=this._calculateTransactionTax(a.amount,a.transaction_percentage,a.fixed),void 0!==a.antecipation_percentage&&void 0!==a.floating&&void 0!==a.installment){for(var g=0;11>=g;g++)c[g]=this._calculateInstallmentValue(a.amount,g+1),d[g]=this._calculateAntecipationPercentage(b,a,g,a.amount),e[g]=this._calculateTotalTax(d[g],b),f[g]=this._calculateLiquidValue(a.amount,e[g]);return{amount:a.amount,transaction_tax:b,antecipation_percentage:d,total_tax:e,liquid_value:f,installment_value:c}}return f=this._calculateLiquidValue(a.amount,b),{amount:a.amount,transaction_tax:b,liquid_value:f}},pricingWithInterest:function(a){for(var b=[],c=[],d=[],e=[],f=[],g=[],h=[],i=1;12>=i;i++)b[i-1]=this._calculateInterestRate(a.amount,a.interest_rate,i),c[i-1]=this._calculateAmount(b[i-1],i),d[i-1]=this._calculateTransactionTax(c[i-1],a.transaction_percentage,a.fixed),e[i-1]=this._calculateAntecipationPercentage(d[i-1],a,i-1,c[i-1]),f[i-1]=this._calculateAntecipationPercentageFromAmount(c[i-1],e[i-1]),g[i-1]=this._calculateTotalTax(e[i-1],d[i-1]),h[i-1]=this._calculateLiquidValue(c[i-1],g[i-1]);return{amount:c,transaction_tax:d,antecipation_percentage:f,total_tax:g,liquid_value:h,installment_value:b}},_calculateAntecipationPercentageFromAmount:function(a,b){return parseFloat(100*b/(a/100)).toFixed(2)},_calculateTransactionTax:function(a,b,c){return parseFloat(((a*(b/100)+c)/100).toFixed(2))},_calculateAntecipationPercentage:function(a,b,c,d){return parseFloat((parseFloat(b.antecipation_percentage/100/30*(30+15*c-b.floating))*parseFloat(d/100-a)).toFixed(2))},_calculateTotalTax:function(a,b){return parseFloat((a+b).toFixed(2))},_calculateLiquidValue:function(a,b){return parseFloat((a/100-parseFloat(b)).toFixed(2))},_calculateInstallmentValue:function(a,b){return parseFloat((a/b/100).toFixed(2))},_calculateInterestRate:function(a,b,c){return parseFloat(1===c?a/100:(this._coefficient(b,c)*(a/100)).toFixed(2))},_calculateAmount:function(a,b){return parseFloat(100*(a*b).toFixed(2))},_coefficient:function(a,b){return parseFloat((a/100/(1-1/Math.pow(a/100+1,b))).toFixed(10))}}}(window),function(a){function b(a){this.number=c.Validator.normalizeCardNumber(a.number),this.cvc=a.cvc,this.expMonth=a.expMonth,this.expYear=a.expYear,this.pubKey=a.pubKey}var c=a.Moip||{};a.Moip=c,b.prototype={hash:function(){if(!(this.pubKey&&this.number&&this.cvc&&this.expMonth&&this.expYear))return null;var a=new JSEncrypt({default_key_size:2048});a.setPublicKey(this.pubKey);var b="number="+this.number+"&";return b+="cvc="+this.cvc+"&",b+="expirationMonth="+this.expMonth+"&",b+="expirationYear="+this.expYear,a.encrypt(b)},isValid:function(){if(!(this.pubKey&&this.number&&this.cvc&&this.expMonth&&this.expYear))return!1;var a=c.Validator.isValid(this.number),b=c.Validator.isSecurityCodeValid(this.number,this.cvc),d=c.Validator.isExpiryDateValid(this.expMonth,this.expYear);return a&&b&&d},cardType:function(){return type=c.Validator.cardType(this.number),type?type.brand:null},_setPubKey:function(a){this.pubKey=a},_setNumber:function(a){this.number=a},_setCvc:function(a){this.cvc=a},_setExpMonth:function(a){this.expMonth=a},_setExpYear:function(a){this.expYear=a}},c.CreditCard=b}(window),function(a){function b(){return this instanceof b?void 0:new b}var c=a.Moip||{};a.Moip=c,b.prototype={_eloBins:["401178","401179","431274","438935","451416","457393","457631","457632","504175","627780","636297","636368"],_eloBinRanges:[[506699,506778],[509e3,509999],[650031,650033],[650035,650051],[650405,650439],[650485,650538],[650541,650598],[650700,650718],[650720,650727],[650901,650920],[651652,651679],[655e3,655019],[655021,655058]],_hiperBins:["637095","637612","637599","637609","637568"],_hipercardBins:["606282","384100","384140","384160"],_masterCardRanges:[222100,272099],_isInEloBinRanges:function(a){for(var b=parseInt(a),c=0;c<this._eloBinRanges.length;c++){var d=this._eloBinRanges[c][0],e=this._eloBinRanges[c][1];if(b>=d&&e>=b)return!0}return!1},_isInMasterCardRanges:function(a){for(var b=parseInt(a),c=0;c<this._masterCardRanges.length;c+=2){var d=this._masterCardRanges[c],e=this._masterCardRanges[c+1];if(b>=d&&e>=b)return!0}return!1},normalizeCardNumber:function(a){return a?(a+="",a.replace(/[\s+|\.|\-]/g,"")):a},isValid:function(a){var c=this.normalizeCardNumber(a),d=b.prototype.cardType(c);if(d){if("HIPERCARD"===d.brand)return!0;for(var e=0,f=2-c.length%2;f<=c.length;f+=2)e+=parseInt(c.charAt(f-1),10);for(f=c.length%2+1;f<c.length;f+=2){var g=2*parseInt(c.charAt(f-1),10);e+=10>g?g:g-9}return e%10===0?!0:!1}return!1},cardType:function(a,b){var c=this,d=this.normalizeCardNumber(a),e=function(a){return a.substring(0,6)},f={VISA:{matches:function(a){return/^4\d{15}$/.test(a)}},MASTERCARD:{matches:function(a){return/^5[1-5]\d{14}$/.test(a)||null!==a&&16==a.length&&c._isInMasterCardRanges(e(a))}},AMEX:{matches:function(a){return/^3[4,7]\d{13}$/.test(a)}},DINERS:{matches:function(a){return/^3[0,6,8]\d{12}$/.test(a)}},HIPERCARD:{matches:function(a){return null!==a&&(16==a.length||19==a.length)&&c._hipercardBins.indexOf(e(a))>-1}},ELO:{matches:function(a){return null!==a&&16==a.length&&(c._eloBins.indexOf(e(a))>-1||c._isInEloBinRanges(e(a)))}},HIPER:{matches:function(a){return null!==a&&a.length>=6&&c._hiperBins.indexOf(e(a))>-1}}},g={VISA:{matches:function(a){return/^4\d{3}\d*$/.test(a)}},MASTERCARD:{matches:function(a){return/^5[1-5]\d{4}\d*$/.test(a)||null!==a&&16==a.length&&c._isInMasterCardRanges(e(a))}},AMEX:{matches:function(a){return/^3[4,7]\d{2}\d*$/.test(a)}},DINERS:{matches:function(a){return/^3(?:0[0-5]|[68][0-9])+\d*$/.test(a)}},HIPERCARD:{matches:function(a){return null!==a&&a.length>=6&&c._hipercardBins.indexOf(e(a))>-1}},ELO:{matches:function(a){return null!==a&&a.length>=6&&(c._eloBins.indexOf(e(a))>-1||c._isInEloBinRanges(e(a)))}},HIPER:{matches:function(a){return null!==a&&a.length>=6&&c._hiperBins.indexOf(e(a))>-1}}};return b&&(f=g),f.ELO.matches(d)?{brand:"ELO"}:f.HIPER.matches(d)?{brand:"HIPER"}:f.VISA.matches(d)?{brand:"VISA"}:f.MASTERCARD.matches(d)?{brand:"MASTERCARD"}:f.AMEX.matches(d)?{brand:"AMEX"}:f.HIPERCARD.matches(d)?{brand:"HIPERCARD"}:f.DINERS.matches(d)?{brand:"DINERS"}:null},isSecurityCodeValid:function(a,b){var d=c.Validator.cardType(a);if(!d)return!1;var e="AMEX"===d.brand?4:3,f=new RegExp("[0-9]{"+e+"}");return b.length===e&&f.test(b)},isExpiryDateValid:function(a,c){return a=parseInt(a,10),c=parseInt(c,10),1>a||a>12?!1:2!==(c+"").length&&4!==(c+"").length?!1:(2===(c+"").length&&(c=c>80?"19"+c:"20"+c),1e3>c||c>=3e3?!1:!b.prototype.isExpiredDate(a,c))},isExpiredDate:function(a,b){var c=new Date,d=("0"+(c.getMonth()+1)).slice(-2),e=c.getFullYear();if(a=("0"+a).slice(-2),2===b.toString().length){if(b>80)return!0;b="20"+b}var f=e+d,g=b+a;return parseInt(g,10)<parseInt(f,10)}},c.Validator=b()}(window);;/*!
 * sweetalert2 v6.6.2
 * Released under the MIT License.
 */
(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.Sweetalert2 = factory());
}(this, (function () { 'use strict';

var defaultParams = {
  title: '',
  titleText: '',
  text: '',
  html: '',
  type: null,
  customClass: '',
  target: 'body',
  animation: true,
  allowOutsideClick: true,
  allowEscapeKey: true,
  allowEnterKey: true,
  showConfirmButton: true,
  showCancelButton: false,
  preConfirm: null,
  confirmButtonText: 'OK',
  confirmButtonColor: '#3085d6',
  confirmButtonClass: null,
  cancelButtonText: 'Cancel',
  cancelButtonColor: '#aaa',
  cancelButtonClass: null,
  buttonsStyling: true,
  reverseButtons: false,
  focusCancel: false,
  showCloseButton: false,
  showLoaderOnConfirm: false,
  imageUrl: null,
  imageWidth: null,
  imageHeight: null,
  imageClass: null,
  timer: null,
  width: 500,
  padding: 20,
  background: '#fff',
  input: null,
  inputPlaceholder: '',
  inputValue: '',
  inputOptions: {},
  inputAutoTrim: true,
  inputClass: null,
  inputAttributes: {},
  inputValidator: null,
  progressSteps: [],
  currentProgressStep: null,
  progressStepsDistance: '40px',
  onOpen: null,
  onClose: null
};

var swalPrefix = 'swal2-';

var prefix = function prefix(items) {
  var result = {};
  for (var i in items) {
    result[items[i]] = swalPrefix + items[i];
  }
  return result;
};

var swalClasses = prefix(['container', 'shown', 'iosfix', 'modal', 'overlay', 'fade', 'show', 'hide', 'noanimation', 'close', 'title', 'content', 'buttonswrapper', 'confirm', 'cancel', 'icon', 'image', 'input', 'file', 'range', 'select', 'radio', 'checkbox', 'textarea', 'inputerror', 'validationerror', 'progresssteps', 'activeprogressstep', 'progresscircle', 'progressline', 'loading', 'styled']);

var iconTypes = prefix(['success', 'warning', 'info', 'question', 'error']);

/*
 * Set hover, active and focus-states for buttons (source: http://www.sitepoint.com/javascript-generate-lighter-darker-color)
 */
var colorLuminance = function colorLuminance(hex, lum) {
  // Validate hex string
  hex = String(hex).replace(/[^0-9a-f]/gi, '');
  if (hex.length < 6) {
    hex = hex[0] + hex[0] + hex[1] + hex[1] + hex[2] + hex[2];
  }
  lum = lum || 0;

  // Convert to decimal and change luminosity
  var rgb = '#';
  for (var i = 0; i < 3; i++) {
    var c = parseInt(hex.substr(i * 2, 2), 16);
    c = Math.round(Math.min(Math.max(0, c + c * lum), 255)).toString(16);
    rgb += ('00' + c).substr(c.length);
  }

  return rgb;
};

var uniqueArray = function uniqueArray(arr) {
  var result = [];
  for (var i in arr) {
    if (result.indexOf(arr[i]) === -1) {
      result.push(arr[i]);
    }
  }
  return result;
};

/* global MouseEvent */

// Remember state in cases where opening and handling a modal will fiddle with it.
var states = {
  previousWindowKeyDown: null,
  previousActiveElement: null,
  previousBodyPadding: null
};

/*
 * Add modal + overlay to DOM
 */
var init = function init(params) {
  if (typeof document === 'undefined') {
    console.error('SweetAlert2 requires document to initialize');
    return;
  }

  var container = document.createElement('div');
  container.className = swalClasses.container;
  container.innerHTML = sweetHTML;

  var targetElement = document.querySelector(params.target);
  if (!targetElement) {
    console.warn('SweetAlert2: Can\'t find the target "' + params.target + '"');
    targetElement = document.body;
  }
  targetElement.appendChild(container);

  var modal = getModal();
  var input = getChildByClass(modal, swalClasses.input);
  var file = getChildByClass(modal, swalClasses.file);
  var range = modal.querySelector('.' + swalClasses.range + ' input');
  var rangeOutput = modal.querySelector('.' + swalClasses.range + ' output');
  var select = getChildByClass(modal, swalClasses.select);
  var checkbox = modal.querySelector('.' + swalClasses.checkbox + ' input');
  var textarea = getChildByClass(modal, swalClasses.textarea);

  input.oninput = function () {
    sweetAlert.resetValidationError();
  };

  input.onkeydown = function (event) {
    setTimeout(function () {
      if (event.keyCode === 13 && params.allowEnterKey) {
        event.stopPropagation();
        sweetAlert.clickConfirm();
      }
    }, 0);
  };

  file.onchange = function () {
    sweetAlert.resetValidationError();
  };

  range.oninput = function () {
    sweetAlert.resetValidationError();
    rangeOutput.value = range.value;
  };

  range.onchange = function () {
    sweetAlert.resetValidationError();
    range.previousSibling.value = range.value;
  };

  select.onchange = function () {
    sweetAlert.resetValidationError();
  };

  checkbox.onchange = function () {
    sweetAlert.resetValidationError();
  };

  textarea.oninput = function () {
    sweetAlert.resetValidationError();
  };

  return modal;
};

/*
 * Manipulate DOM
 */

var sweetHTML = ('\n <div role="dialog" aria-labelledby="' + swalClasses.title + '" aria-describedby="' + swalClasses.content + '" class="' + swalClasses.modal + '" tabindex="-1">\n   <ul class="' + swalClasses.progresssteps + '"></ul>\n   <div class="' + swalClasses.icon + ' ' + iconTypes.error + '">\n     <span class="swal2-x-mark"><span class="swal2-x-mark-line-left"></span><span class="swal2-x-mark-line-right"></span></span>\n   </div>\n   <div class="' + swalClasses.icon + ' ' + iconTypes.question + '">?</div>\n   <div class="' + swalClasses.icon + ' ' + iconTypes.warning + '">!</div>\n   <div class="' + swalClasses.icon + ' ' + iconTypes.info + '">i</div>\n   <div class="' + swalClasses.icon + ' ' + iconTypes.success + '">\n     <div class="swal2-success-circular-line-left"></div>\n     <span class="swal2-success-line-tip"></span> <span class="swal2-success-line-long"></span>\n     <div class="swal2-success-ring"></div> <div class="swal2-success-fix"></div>\n     <div class="swal2-success-circular-line-right"></div>\n   </div>\n   <img class="' + swalClasses.image + '">\n   <h2 class="' + swalClasses.title + '" id="' + swalClasses.title + '"></h2>\n   <div id="' + swalClasses.content + '" class="' + swalClasses.content + '"></div>\n   <input class="' + swalClasses.input + '">\n   <input type="file" class="' + swalClasses.file + '">\n   <div class="' + swalClasses.range + '">\n     <output></output>\n     <input type="range">\n   </div>\n   <select class="' + swalClasses.select + '"></select>\n   <div class="' + swalClasses.radio + '"></div>\n   <label for="' + swalClasses.checkbox + '" class="' + swalClasses.checkbox + '">\n     <input type="checkbox">\n   </label>\n   <textarea class="' + swalClasses.textarea + '"></textarea>\n   <div class="' + swalClasses.validationerror + '"></div>\n   <div class="' + swalClasses.buttonswrapper + '">\n     <button type="button" class="' + swalClasses.confirm + '">OK</button>\n     <button type="button" class="' + swalClasses.cancel + '">Cancel</button>\n   </div>\n   <button type="button" class="' + swalClasses.close + '" aria-label="Close this dialog">&times;</button>\n </div>\n').replace(/(^|\n)\s*/g, '');

var getContainer = function getContainer() {
  return document.body.querySelector('.' + swalClasses.container);
};

var getModal = function getModal() {
  return getContainer() ? getContainer().querySelector('.' + swalClasses.modal) : null;
};

var getIcons = function getIcons() {
  var modal = getModal();
  return modal.querySelectorAll('.' + swalClasses.icon);
};

var elementByClass = function elementByClass(className) {
  return getContainer() ? getContainer().querySelector('.' + className) : null;
};

var getTitle = function getTitle() {
  return elementByClass(swalClasses.title);
};

var getContent = function getContent() {
  return elementByClass(swalClasses.content);
};

var getImage = function getImage() {
  return elementByClass(swalClasses.image);
};

var getButtonsWrapper = function getButtonsWrapper() {
  return elementByClass(swalClasses.buttonswrapper);
};

var getProgressSteps = function getProgressSteps() {
  return elementByClass(swalClasses.progresssteps);
};

var getValidationError = function getValidationError() {
  return elementByClass(swalClasses.validationerror);
};

var getConfirmButton = function getConfirmButton() {
  return elementByClass(swalClasses.confirm);
};

var getCancelButton = function getCancelButton() {
  return elementByClass(swalClasses.cancel);
};

var getCloseButton = function getCloseButton() {
  return elementByClass(swalClasses.close);
};

var getFocusableElements = function getFocusableElements(focusCancel) {
  var buttons = [getConfirmButton(), getCancelButton()];
  if (focusCancel) {
    buttons.reverse();
  }
  var focusableElements = buttons.concat(Array.prototype.slice.call(getModal().querySelectorAll('button, input:not([type=hidden]), textarea, select, a, *[tabindex]:not([tabindex="-1"])')));
  return uniqueArray(focusableElements);
};

var hasClass = function hasClass(elem, className) {
  if (elem.classList) {
    return elem.classList.contains(className);
  }
  return false;
};

var focusInput = function focusInput(input) {
  input.focus();

  // place cursor at end of text in text input
  if (input.type !== 'file') {
    // http://stackoverflow.com/a/2345915/1331425
    var val = input.value;
    input.value = '';
    input.value = val;
  }
};

var addClass = function addClass(elem, className) {
  if (!elem || !className) {
    return;
  }
  var classes = className.split(/\s+/).filter(Boolean);
  classes.forEach(function (className) {
    elem.classList.add(className);
  });
};

var removeClass = function removeClass(elem, className) {
  if (!elem || !className) {
    return;
  }
  var classes = className.split(/\s+/).filter(Boolean);
  classes.forEach(function (className) {
    elem.classList.remove(className);
  });
};

var getChildByClass = function getChildByClass(elem, className) {
  for (var i = 0; i < elem.childNodes.length; i++) {
    if (hasClass(elem.childNodes[i], className)) {
      return elem.childNodes[i];
    }
  }
};

var show = function show(elem, display) {
  if (!display) {
    display = 'block';
  }
  elem.style.opacity = '';
  elem.style.display = display;
};

var hide = function hide(elem) {
  elem.style.opacity = '';
  elem.style.display = 'none';
};

var empty = function empty(elem) {
  while (elem.firstChild) {
    elem.removeChild(elem.firstChild);
  }
};

// borrowed from jqeury $(elem).is(':visible') implementation
var isVisible = function isVisible(elem) {
  return elem.offsetWidth || elem.offsetHeight || elem.getClientRects().length;
};

var removeStyleProperty = function removeStyleProperty(elem, property) {
  if (elem.style.removeProperty) {
    elem.style.removeProperty(property);
  } else {
    elem.style.removeAttribute(property);
  }
};

var fireClick = function fireClick(node) {
  if (!isVisible(node)) {
    return false;
  }

  // Taken from http://www.nonobtrusive.com/2011/11/29/programatically-fire-crossbrowser-click-event-with-javascript/
  // Then fixed for today's Chrome browser.
  if (typeof MouseEvent === 'function') {
    // Up-to-date approach
    var mevt = new MouseEvent('click', {
      view: window,
      bubbles: false,
      cancelable: true
    });
    node.dispatchEvent(mevt);
  } else if (document.createEvent) {
    // Fallback
    var evt = document.createEvent('MouseEvents');
    evt.initEvent('click', false, false);
    node.dispatchEvent(evt);
  } else if (document.createEventObject) {
    node.fireEvent('onclick');
  } else if (typeof node.onclick === 'function') {
    node.onclick();
  }
};

var animationEndEvent = function () {
  var testEl = document.createElement('div');
  var transEndEventNames = {
    'WebkitAnimation': 'webkitAnimationEnd',
    'OAnimation': 'oAnimationEnd oanimationend',
    'msAnimation': 'MSAnimationEnd',
    'animation': 'animationend'
  };
  for (var i in transEndEventNames) {
    if (transEndEventNames.hasOwnProperty(i) && testEl.style[i] !== undefined) {
      return transEndEventNames[i];
    }
  }

  return false;
}();

// Reset previous window keydown handler and focued element
var resetPrevState = function resetPrevState() {
  window.onkeydown = states.previousWindowKeyDown;
  if (states.previousActiveElement && states.previousActiveElement.focus) {
    var x = window.scrollX;
    var y = window.scrollY;
    states.previousActiveElement.focus();
    if (x && y) {
      // IE has no scrollX/scrollY support
      window.scrollTo(x, y);
    }
  }
};

// Measure width of scrollbar
// https://github.com/twbs/bootstrap/blob/master/js/modal.js#L279-L286
var measureScrollbar = function measureScrollbar() {
  var supportsTouch = 'ontouchstart' in window || navigator.msMaxTouchPoints;
  if (supportsTouch) {
    return 0;
  }
  var scrollDiv = document.createElement('div');
  scrollDiv.style.width = '50px';
  scrollDiv.style.height = '50px';
  scrollDiv.style.overflow = 'scroll';
  document.body.appendChild(scrollDiv);
  var scrollbarWidth = scrollDiv.offsetWidth - scrollDiv.clientWidth;
  document.body.removeChild(scrollDiv);
  return scrollbarWidth;
};

// JavaScript Debounce Function
// Simplivied version of https://davidwalsh.name/javascript-debounce-function
var debounce = function debounce(func, wait) {
  var timeout = void 0;
  return function () {
    var later = function later() {
      timeout = null;
      func();
    };
    clearTimeout(timeout);
    timeout = setTimeout(later, wait);
  };
};

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};





















var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var modalParams = _extends({}, defaultParams);
var queue = [];
var swal2Observer = void 0;

/*
 * Set type, text and actions on modal
 */
var setParameters = function setParameters(params) {
  var modal = getModal() || init(params);

  for (var param in params) {
    if (!defaultParams.hasOwnProperty(param) && param !== 'extraParams') {
      console.warn('SweetAlert2: Unknown parameter "' + param + '"');
    }
  }

  // Set modal width
  modal.style.width = typeof params.width === 'number' ? params.width + 'px' : params.width;

  modal.style.padding = params.padding + 'px';
  modal.style.background = params.background;
  var successIconParts = modal.querySelectorAll('[class^=swal2-success-circular-line], .swal2-success-fix');
  for (var i = 0; i < successIconParts.length; i++) {
    successIconParts[i].style.background = params.background;
  }

  var title = getTitle();
  var content = getContent();
  var buttonsWrapper = getButtonsWrapper();
  var confirmButton = getConfirmButton();
  var cancelButton = getCancelButton();
  var closeButton = getCloseButton();

  // Title
  if (params.titleText) {
    title.innerText = params.titleText;
  } else {
    title.innerHTML = params.title.split('\n').join('<br>');
  }

  // Content
  if (params.text || params.html) {
    if (_typeof(params.html) === 'object') {
      content.innerHTML = '';
      if (0 in params.html) {
        for (var _i = 0; _i in params.html; _i++) {
          content.appendChild(params.html[_i].cloneNode(true));
        }
      } else {
        content.appendChild(params.html.cloneNode(true));
      }
    } else if (params.html) {
      content.innerHTML = params.html;
    } else if (params.text) {
      content.textContent = params.text;
    }
    show(content);
  } else {
    hide(content);
  }

  // Close button
  if (params.showCloseButton) {
    show(closeButton);
  } else {
    hide(closeButton);
  }

  // Custom Class
  modal.className = swalClasses.modal;
  if (params.customClass) {
    addClass(modal, params.customClass);
  }

  // Progress steps
  var progressStepsContainer = getProgressSteps();
  var currentProgressStep = parseInt(params.currentProgressStep === null ? sweetAlert.getQueueStep() : params.currentProgressStep, 10);
  if (params.progressSteps.length) {
    show(progressStepsContainer);
    empty(progressStepsContainer);
    if (currentProgressStep >= params.progressSteps.length) {
      console.warn('SweetAlert2: Invalid currentProgressStep parameter, it should be less than progressSteps.length ' + '(currentProgressStep like JS arrays starts from 0)');
    }
    params.progressSteps.forEach(function (step, index) {
      var circle = document.createElement('li');
      addClass(circle, swalClasses.progresscircle);
      circle.innerHTML = step;
      if (index === currentProgressStep) {
        addClass(circle, swalClasses.activeprogressstep);
      }
      progressStepsContainer.appendChild(circle);
      if (index !== params.progressSteps.length - 1) {
        var line = document.createElement('li');
        addClass(line, swalClasses.progressline);
        line.style.width = params.progressStepsDistance;
        progressStepsContainer.appendChild(line);
      }
    });
  } else {
    hide(progressStepsContainer);
  }

  // Icon
  var icons = getIcons();
  for (var _i2 = 0; _i2 < icons.length; _i2++) {
    hide(icons[_i2]);
  }
  if (params.type) {
    var validType = false;
    for (var iconType in iconTypes) {
      if (params.type === iconType) {
        validType = true;
        break;
      }
    }
    if (!validType) {
      console.error('SweetAlert2: Unknown alert type: ' + params.type);
      return false;
    }
    var icon = modal.querySelector('.' + swalClasses.icon + '.' + iconTypes[params.type]);
    show(icon);

    // Animate icon
    if (params.animation) {
      switch (params.type) {
        case 'success':
          addClass(icon, 'swal2-animate-success-icon');
          addClass(icon.querySelector('.swal2-success-line-tip'), 'swal2-animate-success-line-tip');
          addClass(icon.querySelector('.swal2-success-line-long'), 'swal2-animate-success-line-long');
          break;
        case 'error':
          addClass(icon, 'swal2-animate-error-icon');
          addClass(icon.querySelector('.swal2-x-mark'), 'swal2-animate-x-mark');
          break;
        default:
          break;
      }
    }
  }

  // Custom image
  var image = getImage();
  if (params.imageUrl) {
    image.setAttribute('src', params.imageUrl);
    show(image);

    if (params.imageWidth) {
      image.setAttribute('width', params.imageWidth);
    } else {
      image.removeAttribute('width');
    }

    if (params.imageHeight) {
      image.setAttribute('height', params.imageHeight);
    } else {
      image.removeAttribute('height');
    }

    image.className = swalClasses.image;
    if (params.imageClass) {
      addClass(image, params.imageClass);
    }
  } else {
    hide(image);
  }

  // Cancel button
  if (params.showCancelButton) {
    cancelButton.style.display = 'inline-block';
  } else {
    hide(cancelButton);
  }

  // Confirm button
  if (params.showConfirmButton) {
    removeStyleProperty(confirmButton, 'display');
  } else {
    hide(confirmButton);
  }

  // Buttons wrapper
  if (!params.showConfirmButton && !params.showCancelButton) {
    hide(buttonsWrapper);
  } else {
    show(buttonsWrapper);
  }

  // Edit text on cancel and confirm buttons
  confirmButton.innerHTML = params.confirmButtonText;
  cancelButton.innerHTML = params.cancelButtonText;

  // Set buttons to selected background colors
  if (params.buttonsStyling) {
    confirmButton.style.backgroundColor = params.confirmButtonColor;
    cancelButton.style.backgroundColor = params.cancelButtonColor;
  }

  // Add buttons custom classes
  confirmButton.className = swalClasses.confirm;
  addClass(confirmButton, params.confirmButtonClass);
  cancelButton.className = swalClasses.cancel;
  addClass(cancelButton, params.cancelButtonClass);

  // Buttons styling
  if (params.buttonsStyling) {
    addClass(confirmButton, swalClasses.styled);
    addClass(cancelButton, swalClasses.styled);
  } else {
    removeClass(confirmButton, swalClasses.styled);
    removeClass(cancelButton, swalClasses.styled);

    confirmButton.style.backgroundColor = confirmButton.style.borderLeftColor = confirmButton.style.borderRightColor = '';
    cancelButton.style.backgroundColor = cancelButton.style.borderLeftColor = cancelButton.style.borderRightColor = '';
  }

  // CSS animation
  if (params.animation === true) {
    removeClass(modal, swalClasses.noanimation);
  } else {
    addClass(modal, swalClasses.noanimation);
  }
};

/*
 * Animations
 */
var openModal = function openModal(animation, onComplete) {
  var container = getContainer();
  var modal = getModal();

  if (animation) {
    addClass(modal, swalClasses.show);
    addClass(container, swalClasses.fade);
    removeClass(modal, swalClasses.hide);
  } else {
    removeClass(modal, swalClasses.fade);
  }
  show(modal);

  // scrolling is 'hidden' until animation is done, after that 'auto'
  container.style.overflowY = 'hidden';
  if (animationEndEvent && !hasClass(modal, swalClasses.noanimation)) {
    modal.addEventListener(animationEndEvent, function swalCloseEventFinished() {
      modal.removeEventListener(animationEndEvent, swalCloseEventFinished);
      container.style.overflowY = 'auto';
    });
  } else {
    container.style.overflowY = 'auto';
  }

  addClass(document.documentElement, swalClasses.shown);
  addClass(document.body, swalClasses.shown);
  addClass(container, swalClasses.shown);
  fixScrollbar();
  iOSfix();
  states.previousActiveElement = document.activeElement;
  if (onComplete !== null && typeof onComplete === 'function') {
    setTimeout(function () {
      onComplete(modal);
    });
  }
};

var fixScrollbar = function fixScrollbar() {
  // for queues, do not do this more than once
  if (states.previousBodyPadding !== null) {
    return;
  }
  // if the body has overflow
  if (document.body.scrollHeight > window.innerHeight) {
    // add padding so the content doesn't shift after removal of scrollbar
    states.previousBodyPadding = document.body.style.paddingRight;
    document.body.style.paddingRight = measureScrollbar() + 'px';
  }
};

var undoScrollbar = function undoScrollbar() {
  if (states.previousBodyPadding !== null) {
    document.body.style.paddingRight = states.previousBodyPadding;
    states.previousBodyPadding = null;
  }
};

// Fix iOS scrolling http://stackoverflow.com/q/39626302/1331425
var iOSfix = function iOSfix() {
  var iOS = /iPad|iPhone|iPod/.test(navigator.userAgent) && !window.MSStream;
  if (iOS && !hasClass(document.body, swalClasses.iosfix)) {
    var offset = document.body.scrollTop;
    document.body.style.top = offset * -1 + 'px';
    addClass(document.body, swalClasses.iosfix);
  }
};

var undoIOSfix = function undoIOSfix() {
  if (hasClass(document.body, swalClasses.iosfix)) {
    var offset = parseInt(document.body.style.top, 10);
    removeClass(document.body, swalClasses.iosfix);
    document.body.style.top = '';
    document.body.scrollTop = offset * -1;
  }
};

// SweetAlert entry point
var sweetAlert = function sweetAlert() {
  for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  if (args[0] === undefined) {
    console.error('SweetAlert2 expects at least 1 attribute!');
    return false;
  }

  var params = _extends({}, modalParams);

  switch (_typeof(args[0])) {
    case 'string':
      params.title = args[0];
      params.html = args[1];
      params.type = args[2];

      break;

    case 'object':
      _extends(params, args[0]);
      params.extraParams = args[0].extraParams;

      if (params.input === 'email' && params.inputValidator === null) {
        params.inputValidator = function (email) {
          return new Promise(function (resolve, reject) {
            var emailRegex = /^[a-zA-Z0-9.+_-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,6}$/;
            if (emailRegex.test(email)) {
              resolve();
            } else {
              reject('Invalid email address');
            }
          });
        };
      }

      if (params.input === 'url' && params.inputValidator === null) {
        params.inputValidator = function (url) {
          return new Promise(function (resolve, reject) {
            var urlRegex = /^(https?:\/\/)?([\da-z.-]+)\.([a-z.]{2,6})([/\w .-]*)*\/?$/;
            if (urlRegex.test(url)) {
              resolve();
            } else {
              reject('Invalid URL');
            }
          });
        };
      }
      break;

    default:
      console.error('SweetAlert2: Unexpected type of argument! Expected "string" or "object", got ' + _typeof(args[0]));
      return false;
  }

  setParameters(params);

  var container = getContainer();
  var modal = getModal();

  return new Promise(function (resolve, reject) {
    // Close on timer
    if (params.timer) {
      modal.timeout = setTimeout(function () {
        sweetAlert.closeModal(params.onClose);
        reject('timer');
      }, params.timer);
    }

    // Get input element by specified type or, if type isn't specified, by params.input
    var getInput = function getInput(inputType) {
      inputType = inputType || params.input;
      if (!inputType) {
        return null;
      }
      switch (inputType) {
        case 'select':
        case 'textarea':
        case 'file':
          return getChildByClass(modal, swalClasses[inputType]);
        case 'checkbox':
          return modal.querySelector('.' + swalClasses.checkbox + ' input');
        case 'radio':
          return modal.querySelector('.' + swalClasses.radio + ' input:checked') || modal.querySelector('.' + swalClasses.radio + ' input:first-child');
        case 'range':
          return modal.querySelector('.' + swalClasses.range + ' input');
        default:
          return getChildByClass(modal, swalClasses.input);
      }
    };

    // Get the value of the modal input
    var getInputValue = function getInputValue() {
      var input = getInput();
      if (!input) {
        return null;
      }
      switch (params.input) {
        case 'checkbox':
          return input.checked ? 1 : 0;
        case 'radio':
          return input.checked ? input.value : null;
        case 'file':
          return input.files.length ? input.files[0] : null;
        default:
          return params.inputAutoTrim ? input.value.trim() : input.value;
      }
    };

    // input autofocus
    if (params.input) {
      setTimeout(function () {
        var input = getInput();
        if (input) {
          focusInput(input);
        }
      }, 0);
    }

    var confirm = function confirm(value) {
      if (params.showLoaderOnConfirm) {
        sweetAlert.showLoading();
      }

      if (params.preConfirm) {
        params.preConfirm(value, params.extraParams).then(function (preConfirmValue) {
          sweetAlert.closeModal(params.onClose);
          resolve(preConfirmValue || value);
        }, function (error) {
          sweetAlert.hideLoading();
          if (error) {
            sweetAlert.showValidationError(error);
          }
        });
      } else {
        sweetAlert.closeModal(params.onClose);
        resolve(value);
      }
    };

    // Mouse interactions
    var onButtonEvent = function onButtonEvent(event) {
      var e = event || window.event;
      var target = e.target || e.srcElement;
      var confirmButton = getConfirmButton();
      var cancelButton = getCancelButton();
      var targetedConfirm = confirmButton && (confirmButton === target || confirmButton.contains(target));
      var targetedCancel = cancelButton && (cancelButton === target || cancelButton.contains(target));

      switch (e.type) {
        case 'mouseover':
        case 'mouseup':
          if (params.buttonsStyling) {
            if (targetedConfirm) {
              confirmButton.style.backgroundColor = colorLuminance(params.confirmButtonColor, -0.1);
            } else if (targetedCancel) {
              cancelButton.style.backgroundColor = colorLuminance(params.cancelButtonColor, -0.1);
            }
          }
          break;
        case 'mouseout':
          if (params.buttonsStyling) {
            if (targetedConfirm) {
              confirmButton.style.backgroundColor = params.confirmButtonColor;
            } else if (targetedCancel) {
              cancelButton.style.backgroundColor = params.cancelButtonColor;
            }
          }
          break;
        case 'mousedown':
          if (params.buttonsStyling) {
            if (targetedConfirm) {
              confirmButton.style.backgroundColor = colorLuminance(params.confirmButtonColor, -0.2);
            } else if (targetedCancel) {
              cancelButton.style.backgroundColor = colorLuminance(params.cancelButtonColor, -0.2);
            }
          }
          break;
        case 'click':
          // Clicked 'confirm'
          if (targetedConfirm && sweetAlert.isVisible()) {
            sweetAlert.disableButtons();
            if (params.input) {
              var inputValue = getInputValue();

              if (params.inputValidator) {
                sweetAlert.disableInput();
                params.inputValidator(inputValue, params.extraParams).then(function () {
                  sweetAlert.enableButtons();
                  sweetAlert.enableInput();
                  confirm(inputValue);
                }, function (error) {
                  sweetAlert.enableButtons();
                  sweetAlert.enableInput();
                  if (error) {
                    sweetAlert.showValidationError(error);
                  }
                });
              } else {
                confirm(inputValue);
              }
            } else {
              confirm(true);
            }

            // Clicked 'cancel'
          } else if (targetedCancel && sweetAlert.isVisible()) {
            sweetAlert.disableButtons();
            sweetAlert.closeModal(params.onClose);
            reject('cancel');
          }
          break;
        default:
      }
    };

    var buttons = modal.querySelectorAll('button');
    for (var i = 0; i < buttons.length; i++) {
      buttons[i].onclick = onButtonEvent;
      buttons[i].onmouseover = onButtonEvent;
      buttons[i].onmouseout = onButtonEvent;
      buttons[i].onmousedown = onButtonEvent;
    }

    // Closing modal by close button
    getCloseButton().onclick = function () {
      sweetAlert.closeModal(params.onClose);
      reject('close');
    };

    // Closing modal by overlay click
    container.onclick = function (e) {
      if (e.target !== container) {
        return;
      }
      if (params.allowOutsideClick) {
        sweetAlert.closeModal(params.onClose);
        reject('overlay');
      }
    };

    var buttonsWrapper = getButtonsWrapper();
    var confirmButton = getConfirmButton();
    var cancelButton = getCancelButton();

    // Reverse buttons (Confirm on the right side)
    if (params.reverseButtons) {
      confirmButton.parentNode.insertBefore(cancelButton, confirmButton);
    } else {
      confirmButton.parentNode.insertBefore(confirmButton, cancelButton);
    }

    // Focus handling
    var setFocus = function setFocus(index, increment) {
      var focusableElements = getFocusableElements(params.focusCancel);
      // search for visible elements and select the next possible match
      for (var _i3 = 0; _i3 < focusableElements.length; _i3++) {
        index = index + increment;

        // rollover to first item
        if (index === focusableElements.length) {
          index = 0;

          // go to last item
        } else if (index === -1) {
          index = focusableElements.length - 1;
        }

        // determine if element is visible
        var el = focusableElements[index];
        if (isVisible(el)) {
          return el.focus();
        }
      }
    };

    var handleKeyDown = function handleKeyDown(event) {
      var e = event || window.event;
      var keyCode = e.keyCode || e.which;

      if ([9, 13, 32, 27, 37, 38, 39, 40].indexOf(keyCode) === -1) {
        // Don't do work on keys we don't care about.
        return;
      }

      var targetElement = e.target || e.srcElement;

      var focusableElements = getFocusableElements(params.focusCancel);
      var btnIndex = -1; // Find the button - note, this is a nodelist, not an array.
      for (var _i4 = 0; _i4 < focusableElements.length; _i4++) {
        if (targetElement === focusableElements[_i4]) {
          btnIndex = _i4;
          break;
        }
      }

      // TAB
      if (keyCode === 9) {
        if (!e.shiftKey) {
          // Cycle to the next button
          setFocus(btnIndex, 1);
        } else {
          // Cycle to the prev button
          setFocus(btnIndex, -1);
        }
        e.stopPropagation();
        e.preventDefault();

        // ARROWS - switch focus between buttons
      } else if (keyCode === 37 || keyCode === 38 || keyCode === 39 || keyCode === 40) {
        // focus Cancel button if Confirm button is currently focused
        if (document.activeElement === confirmButton && isVisible(cancelButton)) {
          cancelButton.focus();
          // and vice versa
        } else if (document.activeElement === cancelButton && isVisible(confirmButton)) {
          confirmButton.focus();
        }

        // ENTER/SPACE
      } else if (keyCode === 13 || keyCode === 32) {
        if (btnIndex === -1 && params.allowEnterKey) {
          // ENTER/SPACE clicked outside of a button.
          if (params.focusCancel) {
            fireClick(cancelButton, e);
          } else {
            fireClick(confirmButton, e);
          }
          e.stopPropagation();
          e.preventDefault();
        }

        // ESC
      } else if (keyCode === 27 && params.allowEscapeKey === true) {
        sweetAlert.closeModal(params.onClose);
        reject('esc');
      }
    };

    states.previousWindowKeyDown = window.onkeydown;
    window.onkeydown = handleKeyDown;

    // Loading state
    if (params.buttonsStyling) {
      confirmButton.style.borderLeftColor = params.confirmButtonColor;
      confirmButton.style.borderRightColor = params.confirmButtonColor;
    }

    /**
     * Show spinner instead of Confirm button and disable Cancel button
     */
    sweetAlert.showLoading = sweetAlert.enableLoading = function () {
      show(buttonsWrapper);
      show(confirmButton, 'inline-block');
      addClass(buttonsWrapper, swalClasses.loading);
      addClass(modal, swalClasses.loading);
      confirmButton.disabled = true;
      cancelButton.disabled = true;
    };

    /**
     * Show spinner instead of Confirm button and disable Cancel button
     */
    sweetAlert.hideLoading = sweetAlert.disableLoading = function () {
      if (!params.showConfirmButton) {
        hide(confirmButton);
        if (!params.showCancelButton) {
          hide(getButtonsWrapper());
        }
      }
      removeClass(buttonsWrapper, swalClasses.loading);
      removeClass(modal, swalClasses.loading);
      confirmButton.disabled = false;
      cancelButton.disabled = false;
    };

    sweetAlert.getTitle = function () {
      return getTitle();
    };
    sweetAlert.getContent = function () {
      return getContent();
    };
    sweetAlert.getInput = function () {
      return getInput();
    };
    sweetAlert.getImage = function () {
      return getImage();
    };
    sweetAlert.getButtonsWrapper = function () {
      return getButtonsWrapper();
    };
    sweetAlert.getConfirmButton = function () {
      return getConfirmButton();
    };
    sweetAlert.getCancelButton = function () {
      return getCancelButton();
    };

    sweetAlert.enableButtons = function () {
      confirmButton.disabled = false;
      cancelButton.disabled = false;
    };

    sweetAlert.disableButtons = function () {
      confirmButton.disabled = true;
      cancelButton.disabled = true;
    };

    sweetAlert.enableConfirmButton = function () {
      confirmButton.disabled = false;
    };

    sweetAlert.disableConfirmButton = function () {
      confirmButton.disabled = true;
    };

    sweetAlert.enableInput = function () {
      var input = getInput();
      if (!input) {
        return false;
      }
      if (input.type === 'radio') {
        var radiosContainer = input.parentNode.parentNode;
        var radios = radiosContainer.querySelectorAll('input');
        for (var _i5 = 0; _i5 < radios.length; _i5++) {
          radios[_i5].disabled = false;
        }
      } else {
        input.disabled = false;
      }
    };

    sweetAlert.disableInput = function () {
      var input = getInput();
      if (!input) {
        return false;
      }
      if (input && input.type === 'radio') {
        var radiosContainer = input.parentNode.parentNode;
        var radios = radiosContainer.querySelectorAll('input');
        for (var _i6 = 0; _i6 < radios.length; _i6++) {
          radios[_i6].disabled = true;
        }
      } else {
        input.disabled = true;
      }
    };

    // Set modal min-height to disable scrolling inside the modal
    sweetAlert.recalculateHeight = debounce(function () {
      var modal = getModal();
      if (!modal) {
        return;
      }
      var prevState = modal.style.display;
      modal.style.minHeight = '';
      show(modal);
      modal.style.minHeight = modal.scrollHeight + 1 + 'px';
      modal.style.display = prevState;
    }, 50);

    // Show block with validation error
    sweetAlert.showValidationError = function (error) {
      var validationError = getValidationError();
      validationError.innerHTML = error;
      show(validationError);

      var input = getInput();
      if (input) {
        focusInput(input);
        addClass(input, swalClasses.inputerror);
      }
    };

    // Hide block with validation error
    sweetAlert.resetValidationError = function () {
      var validationError = getValidationError();
      hide(validationError);
      sweetAlert.recalculateHeight();

      var input = getInput();
      if (input) {
        removeClass(input, swalClasses.inputerror);
      }
    };

    sweetAlert.getProgressSteps = function () {
      return params.progressSteps;
    };

    sweetAlert.setProgressSteps = function (progressSteps) {
      params.progressSteps = progressSteps;
      setParameters(params);
    };

    sweetAlert.showProgressSteps = function () {
      show(getProgressSteps());
    };

    sweetAlert.hideProgressSteps = function () {
      hide(getProgressSteps());
    };

    sweetAlert.enableButtons();
    sweetAlert.hideLoading();
    sweetAlert.resetValidationError();

    // inputs
    var inputTypes = ['input', 'file', 'range', 'select', 'radio', 'checkbox', 'textarea'];
    var input = void 0;
    for (var _i7 = 0; _i7 < inputTypes.length; _i7++) {
      var inputClass = swalClasses[inputTypes[_i7]];
      var inputContainer = getChildByClass(modal, inputClass);
      input = getInput(inputTypes[_i7]);

      // set attributes
      if (input) {
        for (var j in input.attributes) {
          if (input.attributes.hasOwnProperty(j)) {
            var attrName = input.attributes[j].name;
            if (attrName !== 'type' && attrName !== 'value') {
              input.removeAttribute(attrName);
            }
          }
        }
        for (var attr in params.inputAttributes) {
          input.setAttribute(attr, params.inputAttributes[attr]);
        }
      }

      // set class
      inputContainer.className = inputClass;
      if (params.inputClass) {
        addClass(inputContainer, params.inputClass);
      }

      hide(inputContainer);
    }

    var populateInputOptions = void 0;
    switch (params.input) {
      case 'text':
      case 'email':
      case 'password':
      case 'number':
      case 'tel':
      case 'url':
        input = getChildByClass(modal, swalClasses.input);
        input.value = params.inputValue;
        input.placeholder = params.inputPlaceholder;
        input.type = params.input;
        show(input);
        break;
      case 'file':
        input = getChildByClass(modal, swalClasses.file);
        input.placeholder = params.inputPlaceholder;
        input.type = params.input;
        show(input);
        break;
      case 'range':
        var range = getChildByClass(modal, swalClasses.range);
        var rangeInput = range.querySelector('input');
        var rangeOutput = range.querySelector('output');
        rangeInput.value = params.inputValue;
        rangeInput.type = params.input;
        rangeOutput.value = params.inputValue;
        show(range);
        break;
      case 'select':
        var select = getChildByClass(modal, swalClasses.select);
        select.innerHTML = '';
        if (params.inputPlaceholder) {
          var placeholder = document.createElement('option');
          placeholder.innerHTML = params.inputPlaceholder;
          placeholder.value = '';
          placeholder.disabled = true;
          placeholder.selected = true;
          select.appendChild(placeholder);
        }
        populateInputOptions = function populateInputOptions(inputOptions) {
          for (var optionValue in inputOptions) {
            var option = document.createElement('option');
            option.value = optionValue;
            option.innerHTML = inputOptions[optionValue];
            if (params.inputValue === optionValue) {
              option.selected = true;
            }
            select.appendChild(option);
          }
          show(select);
          select.focus();
        };
        break;
      case 'radio':
        var radio = getChildByClass(modal, swalClasses.radio);
        radio.innerHTML = '';
        populateInputOptions = function populateInputOptions(inputOptions) {
          for (var radioValue in inputOptions) {
            var radioInput = document.createElement('input');
            var radioLabel = document.createElement('label');
            var radioLabelSpan = document.createElement('span');
            radioInput.type = 'radio';
            radioInput.name = swalClasses.radio;
            radioInput.value = radioValue;
            if (params.inputValue === radioValue) {
              radioInput.checked = true;
            }
            radioLabelSpan.innerHTML = inputOptions[radioValue];
            radioLabel.appendChild(radioInput);
            radioLabel.appendChild(radioLabelSpan);
            radioLabel.for = radioInput.id;
            radio.appendChild(radioLabel);
          }
          show(radio);
          var radios = radio.querySelectorAll('input');
          if (radios.length) {
            radios[0].focus();
          }
        };
        break;
      case 'checkbox':
        var checkbox = getChildByClass(modal, swalClasses.checkbox);
        var checkboxInput = getInput('checkbox');
        checkboxInput.type = 'checkbox';
        checkboxInput.value = 1;
        checkboxInput.id = swalClasses.checkbox;
        checkboxInput.checked = Boolean(params.inputValue);
        var label = checkbox.getElementsByTagName('span');
        if (label.length) {
          checkbox.removeChild(label[0]);
        }
        label = document.createElement('span');
        label.innerHTML = params.inputPlaceholder;
        checkbox.appendChild(label);
        show(checkbox);
        break;
      case 'textarea':
        var textarea = getChildByClass(modal, swalClasses.textarea);
        textarea.value = params.inputValue;
        textarea.placeholder = params.inputPlaceholder;
        show(textarea);
        break;
      case null:
        break;
      default:
        console.error('SweetAlert2: Unexpected type of input! Expected "text", "email", "password", "number", "tel", "select", "radio", "checkbox", "textarea", "file" or "url", got "' + params.input + '"');
        break;
    }

    if (params.input === 'select' || params.input === 'radio') {
      if (params.inputOptions instanceof Promise) {
        sweetAlert.showLoading();
        params.inputOptions.then(function (inputOptions) {
          sweetAlert.hideLoading();
          populateInputOptions(inputOptions);
        });
      } else if (_typeof(params.inputOptions) === 'object') {
        populateInputOptions(params.inputOptions);
      } else {
        console.error('SweetAlert2: Unexpected type of inputOptions! Expected object or Promise, got ' + _typeof(params.inputOptions));
      }
    }

    openModal(params.animation, params.onOpen);

    // Focus the first element (input or button)
    if (params.allowEnterKey) {
      setFocus(-1, 1);
    } else {
      if (document.activeElement) {
        document.activeElement.blur();
      }
    }

    // fix scroll
    getContainer().scrollTop = 0;

    // Observe changes inside the modal and adjust height
    if (typeof MutationObserver !== 'undefined' && !swal2Observer) {
      swal2Observer = new MutationObserver(sweetAlert.recalculateHeight);
      swal2Observer.observe(modal, { childList: true, characterData: true, subtree: true });
    }
  });
};

/*
 * Global function to determine if swal2 modal is shown
 */
sweetAlert.isVisible = function () {
  return !!getModal();
};

/*
 * Global function for chaining sweetAlert modals
 */
sweetAlert.queue = function (steps) {
  queue = steps;
  var resetQueue = function resetQueue() {
    queue = [];
    document.body.removeAttribute('data-swal2-queue-step');
  };
  var queueResult = [];
  return new Promise(function (resolve, reject) {
    (function step(i, callback) {
      if (i < queue.length) {
        document.body.setAttribute('data-swal2-queue-step', i);

        sweetAlert(queue[i]).then(function (result) {
          queueResult.push(result);
          step(i + 1, callback);
        }, function (dismiss) {
          resetQueue();
          reject(dismiss);
        });
      } else {
        resetQueue();
        resolve(queueResult);
      }
    })(0);
  });
};

/*
 * Global function for getting the index of current modal in queue
 */
sweetAlert.getQueueStep = function () {
  return document.body.getAttribute('data-swal2-queue-step');
};

/*
 * Global function for inserting a modal to the queue
 */
sweetAlert.insertQueueStep = function (step, index) {
  if (index && index < queue.length) {
    return queue.splice(index, 0, step);
  }
  return queue.push(step);
};

/*
 * Global function for deleting a modal from the queue
 */
sweetAlert.deleteQueueStep = function (index) {
  if (typeof queue[index] !== 'undefined') {
    queue.splice(index, 1);
  }
};

/*
 * Global function to close sweetAlert
 */
sweetAlert.close = sweetAlert.closeModal = function (onComplete) {
  var container = getContainer();
  var modal = getModal();
  if (!modal) {
    return;
  }
  removeClass(modal, swalClasses.show);
  addClass(modal, swalClasses.hide);
  clearTimeout(modal.timeout);

  resetPrevState();

  var removeModalAndResetState = function removeModalAndResetState() {
    if (container.parentNode) {
      container.parentNode.removeChild(container);
    }
    removeClass(document.documentElement, swalClasses.shown);
    removeClass(document.body, swalClasses.shown);
    undoScrollbar();
    undoIOSfix();
  };

  // If animation is supported, animate
  if (animationEndEvent && !hasClass(modal, swalClasses.noanimation)) {
    modal.addEventListener(animationEndEvent, function swalCloseEventFinished() {
      modal.removeEventListener(animationEndEvent, swalCloseEventFinished);
      if (hasClass(modal, swalClasses.hide)) {
        removeModalAndResetState();
      }
    });
  } else {
    // Otherwise, remove immediately
    removeModalAndResetState();
  }
  if (onComplete !== null && typeof onComplete === 'function') {
    setTimeout(function () {
      onComplete(modal);
    });
  }
};

/*
 * Global function to click 'Confirm' button
 */
sweetAlert.clickConfirm = function () {
  return getConfirmButton().click();
};

/*
 * Global function to click 'Cancel' button
 */
sweetAlert.clickCancel = function () {
  return getCancelButton().click();
};

/**
 * Set default params for each popup
 * @param {Object} userParams
 */
sweetAlert.setDefaults = function (userParams) {
  if (!userParams || (typeof userParams === 'undefined' ? 'undefined' : _typeof(userParams)) !== 'object') {
    return console.error('SweetAlert2: the argument for setDefaults() is required and has to be a object');
  }

  for (var param in userParams) {
    if (!defaultParams.hasOwnProperty(param) && param !== 'extraParams') {
      console.warn('SweetAlert2: Unknown parameter "' + param + '"');
      delete userParams[param];
    }
  }

  _extends(modalParams, userParams);
};

/**
 * Reset default params for each popup
 */
sweetAlert.resetDefaults = function () {
  modalParams = _extends({}, defaultParams);
};

sweetAlert.noop = function () {};

sweetAlert.version = '6.6.2';

sweetAlert.default = sweetAlert;

return sweetAlert;

})));
if (window.Sweetalert2) window.sweetAlert = window.swal = window.Sweetalert2;
;MONSTER( 'MOIP.Application', function(Model, $, utils) {

	var createNames = [
		'HandleCheckoutFields'
	];

	Model.init = function(container) {
		MOIP.BuildComponents.create( container );
		MOIP.BuildCreate.init( container, createNames );
		Model.setTabs();
		Model.copyBilletLinecode();
	};

	Model.setTabs = function() {
		$( '.woocommerce-tabs .panel' ).not( ':eq(0)' ).attr( 'style', 'display: none;' );
		$( '.woocommerce-tabs .panel:eq(0)' ).attr( 'style', 'display: block !important;' );

		$( '.woocommerce-tabs ul.tabs li a' ).on( 'click', function( e ) {
			e.preventDefault();

			var tab 		 = $( this )
			  , tabs_wrapper = tab.closest( '.woocommerce-tabs' )
			;

			$( 'ul.tabs li', tabs_wrapper).removeClass( 'active' );
			$( 'div.panel', tabs_wrapper).attr( 'style', 'display: none;' );
			$( 'div' + tab.attr( 'href' ) ).attr( 'style', 'display: block !important;' );
			tab.parent().addClass( 'active' );
		});
	};

	Model.copyBilletLinecode = function() {
	    var clipboard = new Clipboard( '#clipboard-linecode-btn' );

	    clipboard.on( 'success', function(e) {
	    	var element = $( e.trigger );

	        element.addClass( 'success' ).text( element.data( 'success-text' ) );
	    });

	    clipboard.on( 'error', function(e) {

	    });
	};

});;MONSTER( 'MOIP.Components.CheckoutMoip', function(Model, $, utils) {

	Model.fn.start = function() {
		this.init();
	};

	Model.fn.init = function() {
		this.addEventListener();
	};

	Model.fn.addEventListener = function() {
		this.click( 'payment-link' );
	};

	Model.fn._onClickPaymentLink = function(event) {
		this.elements.tabs.html( utils.getSpinner() );

		window.open( event.currentTarget.href, '_blank' );

		this.ajax({
			data : {
				action      : 'RmSLgKecpN',
				security    : this.data.nonce,
				returnUrl   : this.data.returnUrl,
				paymentType : event.currentTarget.dataset.paymentType,
				order       : this.data.order
			}
		});

		event.preventDefault();
	};

	Model.fn._done = function(response) {
		if ( !response.success ) {
			return;
		}

		window.location.href = response.data.redirectUrl;
	};

	Model.fn._fail = function(jqXHR, textStatus, errorThrown) {

	};

});;MONSTER( 'MOIP.Components.CheckoutTransparent', function(Model, $, utils) {

	var CLASS_HIDE = 'wc-moip-hide-field';

	Model.fn.start = function() {
		this.init();
	};

	Model.fn.init = function() {
		this.setCreditCard();
		this.addEventListener();
		this.checkStoredCC();

		this.messages = MOIPGlobalVars.messages;
	};

	Model.fn.addEventListener = function() {
		this.$el.on( 'submit', this._onSubmit.bind(this) );
		this.click( 'tab' );
		this.click( 'change-cc' );

		if ( typeof $().select2 === 'function' ) {
			this._select2();
		}
	};

	Model.fn._select2 = function() {
		this.$el.byAction( 'select2' ).select2( {
			width                   : '200px',
			minimumResultsForSearch : 20
		});
	};

	Model.fn.setCreditCard = function() {
		if ( this.elements.cardNumber ) {
			this.elements.cardNumber.payment( 'formatCardNumber' );
			this.elements.cardExpiry.payment( 'formatCardExpiry' );
			this.elements.cardCvc.payment( 'formatCardCVC' );
		}
	};

	Model.fn.checkCreditCardData = function() {
		this.expiry     = this.elements.cardExpiry.val();
		this.cardNumber = this.elements.cardNumber.val();
		this.cardCvc    = this.elements.cardCvc.val();

		if ( $.trim( this.expiry ) && $.trim( this.cardNumber ) && $.trim( this.cardCvc ) ) {
			return true;
		}

		return false;
	};

	Model.fn.setCreditCardHash = function() {
		if ( this.isStoredCC() && !this.elements.fieldsCcData.is( ':visible' ) ) {
			return true;
		}

		if ( !this.checkCreditCardData() ) {
			this.failMessage( this.messages.failRequiredFieldsCreditCard );
		}

		var expPieces = this.expiry.split( '/' )
		  , month  	  = expPieces[0]
		  , year      = expPieces[1]
		;

		var cc = new Moip.CreditCard({
			number   : this.cardNumber,
			cvc      : this.cardCvc,
			expMonth : month.trim(),
			expYear  : year.trim(),
			pubKey   : this.elements.publicKey.val()
		});

		if ( ! cc.isValid() ) {
			this.failMessage( this.messages.failDefaultTextCreditCard );
			return false;
		}

		if ( this.data.encrypt ) {
			this.elements.hash.val( cc.hash() );
		}

		return true;
	};

	Model.fn._onSubmit = function(e) {
		e.preventDefault();

		var paymentMethod = $( '[name=payment_method]:checked' ).val();

		if ( paymentMethod == 'payCreditCard' ) {
			if ( ! this.setCreditCardHash() ) {
				return;
			}
		}

		var self = this;

		swal({
			title: this.messages.processingTitle,
			text: this.messages.processingTextWaiting,
			allowOutsideClick : false,
			onOpen: function () {
		    	swal.showLoading();
			    self.ajax({
					data: {
						action  : 'N7yAgMU7JJ',
						order   : self.data.order,
						encrypt : self.data.encrypt,
						fields  : self.$el.serializeArray()
					}
				});
			}
		});
	};

	Model.fn._onClickTab = function(e) {
		this.elements[ e.currentTarget.dataset.ref ].click();
	};

	Model.fn._onClickChangeCc = function(event) {
		this.elements.storedCcInfo.toggleClass( CLASS_HIDE );
		this.elements.fieldsCcData.toggleClass( CLASS_HIDE );
	};

	Model.fn._done = function(response) {
		if ( ! response.success ) {
			this.failMessage( response.data );
		} else {
			this.successMessage();
		}
	};

	Model.fn._fail = function(jqXHR, textStatus, errorThrown) {
		this.failMessage();
	};

	Model.fn.failMessage = function(message) {
		var defaultText = this.messages.failDefaultText;

		swal({
			type : 'error',
			html : message || defaultText
		});
	};

	Model.fn.successMessage = function() {
		var self = this;

		swal({
			type : 'success',
			html: this.messages.processingTextCompleted,
			allowOutsideClick : false
		}).then(function(){
			window.location.href = self.data.returnUrl;
		});
	};

	Model.fn.isStoredCC = function() {
		return ( this.data.storeCreditCard === 1 );
	};

	Model.fn.checkStoredCC = function() {
		if ( this.isStoredCC() && this.elements.cardNumber ) {
			this.elements.storedCcInfo.removeClass( CLASS_HIDE );
			this.elements.fieldsCcData.addClass( CLASS_HIDE );
			this.elements.oldCcInfo.removeClass( CLASS_HIDE );
		}
	};

});;MONSTER( 'MOIP.HandleCheckoutFields', function(Model, $, utils) {

	Model.create = function() {
		if ( ! window.wcbcf_public_params ) {
			this.init();
			this.addEventListener();

			this.personType.trigger( 'change' );
		}
	};

	Model.init = function() {
		this.company    = $( '#billing_company_field' );
		this.cnpj       = $( '#billing_cnpj_field' );
		this.cpf        = $( '#billing_cpf_field' );
		this.personType = $( '#billing_persontype' );

		this.setSelect2();
		this.hideElements();
		this.setRequired();
		this.setMask();
	};

	Model.addEventListener = function() {
		this.personType.on( 'change', this.onChange.bind(this) );
	};

	Model.onChange = function(e) {
		var selectedOption = e.currentTarget.value;

		this.company.hide();
		this.cnpj.hide();
		this.cpf.hide();

		if ( selectedOption == 1 ) {
			this.cpf.show();
		}

		if ( selectedOption == 2 ) {
			this.company.show();
			this.cnpj.show();
		}
	};

	Model.hideElements = function() {
		this.company.hide();
		this.cnpj.hide();
		this.cpf.hide();
	};

	Model.setRequired = function() {
		$( '.person-type-field' ).addClass( 'validate-required' );
		$( '.person-type-field label' ).append( ' <abbr class="required">*</abbr>' );
	};

	Model.setMask = function() {
		$( '#billing_cpf' ).mask( '000.000.000-00' );
		$( '#billing_cnpj' ).mask( '00.000.000/0000-00' );
		$( '#billing_birthdate' ).mask( '00/00/0000', { placeholder: '__/__/____' } );
	};

	Model.setSelect2 = function() {
		this.personType.select2();
	};

});;jQuery(function($) {
	var context = $( 'body' );

	MOIP.vars = {
		body : context
	};

	MOIP.Application.init.apply( null, [context] );
});